import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.squid import Squid
from lobstrio.models.run import Run, RunStats
from lobstrio.models.task import Task, TaskStatus


runner = CliRunner()

SQUIDS = [
    Squid(
        id="squid1abc123def456", name="My Squid", crawler="c1",
        crawler_name="Google Maps", is_active=True, is_ready=True,
        concurrency=1, to_complete=None, last_run_status=None,
        last_run_at=None, total_runs=0, export_unique_results=False,
        params={},
    ),
]

FULL_RUN_HASH = "b" * 32


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client():
    mock = MagicMock()
    mock.squids.list.return_value = SQUIDS
    return mock


class TestRunStart:
    def test_start_run(self):
        mock = _mock_client()
        mock.runs.start.return_value = Run(
            id=FULL_RUN_HASH, status="running", total_results=0,
            total_unique_results=0, duration=0, credit_used=0,
            origin="api", done_reason=None, done_reason_desc=None,
            export_done=False, started_at="2025-01-01T00:00:00Z", ended_at=None,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "start", "My Squid"])
        assert result.exit_code == 0
        assert "Started" in result.output

    def test_start_json_mode(self):
        mock = _mock_client()
        mock.runs.start.return_value = Run(
            id=FULL_RUN_HASH, status="running", total_results=0,
            total_unique_results=0, duration=0, credit_used=0,
            origin="api", done_reason=None, done_reason_desc=None,
            export_done=False, started_at=None, ended_at=None,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "run", "start", "My Squid"])
        assert result.exit_code == 0
        assert FULL_RUN_HASH in result.output


class TestRunLs:
    def test_list_runs(self):
        mock = _mock_client()
        mock.runs.list.return_value = [
            Run(
                id=FULL_RUN_HASH, status="finished", total_results=100,
                total_unique_results=95, duration=120.5, credit_used=50,
                origin="api", done_reason="completed", done_reason_desc=None,
                export_done=True, started_at="2025-01-01", ended_at="2025-01-01",
            ),
        ]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "ls", "My Squid"])
        assert result.exit_code == 0
        assert "finis" in result.output

    def test_list_runs_duration_formatting(self):
        mock = _mock_client()
        mock.runs.list.return_value = [
            Run(
                id=FULL_RUN_HASH, status="finished", total_results=0,
                total_unique_results=0, duration=0, credit_used=0,
                origin="api", done_reason=None, done_reason_desc=None,
                export_done=False, started_at=None, ended_at=None,
            ),
        ]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "ls", "My Squid"])
        assert result.exit_code == 0


class TestRunShow:
    def test_show_run(self):
        mock = _mock_client()
        mock.runs.get.return_value = Run(
            id=FULL_RUN_HASH, status="finished", total_results=100,
            total_unique_results=95, duration=120, credit_used=50,
            origin="api", done_reason="completed", done_reason_desc=None,
            export_done=True, started_at="2025-01-01T00:00:00Z",
            ended_at="2025-01-01T00:02:00Z",
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "show", FULL_RUN_HASH])
        assert result.exit_code == 0
        assert "finished" in result.output

    def test_show_partial_hash_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "show", "abc123"])
        assert result.exit_code != 0


class TestRunStats:
    def test_run_stats(self):
        mock = _mock_client()
        mock.runs.stats.return_value = RunStats(
            percent_done="100%", total_tasks=10, total_tasks_done=10,
            total_tasks_left=0, total_results=500, duration=60,
            eta="0s", current_task=None, is_done=True,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "stats", FULL_RUN_HASH])
        assert result.exit_code == 0
        assert "100%" in result.output

    def test_stats_partial_hash_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "stats", "short"])
        assert result.exit_code != 0


class TestRunTasks:
    def test_list_run_tasks(self):
        mock = _mock_client()
        mock.runs.tasks.return_value = [
            Task(
                id="t" * 32, is_active=True,
                params={"url": "https://example.com"},
                status=TaskStatus(status="done", total_results=10, total_pages=1,
                                  done_reason="completed", has_errors=False),
                created_at=None,
            ),
        ]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "tasks", FULL_RUN_HASH])
        assert result.exit_code == 0
        assert "done" in result.output

    def test_tasks_partial_hash_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "tasks", "short"])
        assert result.exit_code != 0


class TestRunAbort:
    def test_abort_run(self):
        mock = _mock_client()
        mock.runs.abort.return_value = {"status": "aborted"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "abort", FULL_RUN_HASH])
        assert result.exit_code == 0
        assert "Aborted" in result.output

    def test_abort_partial_hash_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "abort", "short"])
        assert result.exit_code != 0


class TestRunDownload:
    def test_download_run(self):
        mock = _mock_client()
        mock.runs.download.return_value = None
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "download", FULL_RUN_HASH])
        assert result.exit_code == 0
        assert "Downloaded" in result.output
        mock.runs.download.assert_called_once()

    def test_download_no_url_fails(self):
        mock = _mock_client()
        mock.runs.download.side_effect = KeyError("s3")
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "download", FULL_RUN_HASH])
        assert result.exit_code == 1

    def test_download_partial_hash_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["run", "download", "short"])
        assert result.exit_code != 0



from __future__ import annotations

import os
import sys
from pathlib import Path

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib
import tomli_w


def get_config_dir() -> Path:
    xdg = os.environ.get("XDG_CONFIG_HOME")
    if xdg:
        base = Path(xdg)
    else:
        base = Path.home() / ".config"
    return base / "lobstr"


def get_config_path() -> Path:
    return get_config_dir() / "config.toml"


def load_config() -> dict:
    path = get_config_path()
    if not path.exists():
        return {}
    with open(path, "rb") as f:
        return tomllib.load(f)


def _save_config(cfg: dict) -> None:
    path = get_config_path()
    path.parent.mkdir(parents=True, exist_ok=True)
    with open(path, "wb") as f:
        tomli_w.dump(cfg, f)


def save_token(token: str) -> None:
    cfg = load_config()
    cfg.setdefault("auth", {})["token"] = token
    _save_config(cfg)


def get_token(override: str | None = None) -> str | None:
    if override:
        return override
    env = os.environ.get("LOBSTR_TOKEN")
    if env:
        return env
    cfg = load_config()
    return cfg.get("auth", {}).get("token")


def save_alias(name: str, hash_value: str) -> None:
    cfg = load_config()
    cfg.setdefault("aliases", {})[name] = hash_value
    _save_config(cfg)


def resolve_alias(value: str) -> str:
    if not value.startswith("@"):
        return value
    name = value[1:]
    cfg = load_config()
    aliases = cfg.get("aliases", {})
    if name in aliases:
        return aliases[name]
    return value



from __future__ import annotations

from dataclasses import asdict
import typer

from lobstr_cli.display import print_json, print_table, print_detail_grouped, print_info

crawlers_app = typer.Typer(no_args_is_help=True)


@crawlers_app.command("ls")
def list_crawlers():
    """List all available crawlers."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    crawlers = client.crawlers.list()
    if _state.get("json"):
        print_json([asdict(c) for c in crawlers])
        return
    rows = []
    for c in crawlers:
        status = ""
        if c.has_issues:
            status = "[yellow]issues[/]"
        elif not c.is_available:
            status = "[red]unavailable[/]"
        elif c.is_premium:
            status = "[cyan]premium[/]"
        rows.append([
            c.name,
            c.slug,
            c.id[:12],
            str(c.credits_per_row or "?"),
            str(c.max_concurrency),
            "yes" if c.account else "no",
            status,
        ])
    print_table(["Name", "Slug", "Hash", "Credits/Row", "Max Conc.", "Needs Account", "Status"], rows)


@crawlers_app.command("show")
def show_crawler(crawler: str = typer.Argument(..., help="Crawler slug, hash, or name")):
    """Show crawler details."""
    from lobstr_cli.cli import get_client, _state
    from lobstr_cli.resolve import resolve_crawler
    client = get_client()
    all_crawlers = client.crawlers.list()
    crawler_id = resolve_crawler(crawler, all_crawlers)
    data = client.crawlers.get(crawler_id)
    if _state.get("json"):
        print_json(asdict(data))
        return
    status = "available"
    if data.has_issues:
        status = "issues"
    elif not data.is_available:
        status = "unavailable"
    elif data.is_premium:
        status = "premium"
    sections = [
        (None, [
            ("Name", data.name),
            ("Slug", data.slug),
            ("Hash", data.id),
            ("Description", data.description),
            ("Status", status),
            ("Rank", data.rank),
        ]),
        ("Credits & Limits", [
            ("Credits/Row", data.credits_per_row),
            ("Credits/Email", data.credits_per_email),
            ("Max Concurrency", data.max_concurrency),
        ]),
        ("Flags", [
            ("Needs Account", "yes" if data.account else "no"),
            ("Email Verification", data.has_email_verification),
            ("Public", data.is_public),
            ("Premium", data.is_premium),
        ]),
    ]
    ws = data.default_worker_stats
    if ws:
        sections.append(("Worker Stats", [
            ("Success Ratio", ws.get("success_ratio")),
            ("Rate (min-max)", f"{ws.get('min_rate_per_worker', '?')}-{ws.get('max_rate_per_worker', '?')}"),
        ]))
    ews = data.email_worker_stats
    if ews:
        sections.append(("Email Worker Stats", [
            ("Success Ratio", ews.get("success_ratio")),
            ("Rate (min-max)", f"{ews.get('min_rate_per_worker', '?')}-{ews.get('max_rate_per_worker', '?')}"),
        ]))
    if data.result_fields:
        sections.append(("Result Fields", [
            ("Fields", ", ".join(data.result_fields)),
        ]))
    print_detail_grouped(sections)
    if data.input_params:
        print_info("\nInput Parameters:")
        input_rows = []
        for inp in data.input_params:
            input_rows.append([
                inp.get("name", ""),
                inp.get("level", ""),
                inp.get("type", ""),
                "yes" if inp.get("required") == "true" else "no",
                str(inp.get("default", "")),
            ])
        print_table(["Name", "Level", "Type", "Required", "Default"], input_rows)


@crawlers_app.command("params")
def crawler_params(crawler: str = typer.Argument(..., help="Crawler hash or prefix")):
    """Show parameters for a crawler."""
    from lobstr_cli.cli import get_client, _state
    from lobstr_cli.resolve import resolve_crawler
    client = get_client()
    all_crawlers = client.crawlers.list()
    crawler_id = resolve_crawler(crawler, all_crawlers)
    params = client.crawlers.params(crawler_id)
    if _state.get("json"):
        print_json(asdict(params))
        return
    if params.task_params:
        print_info("Task-level parameters:")
        rows = []
        for name, spec in params.task_params.items():
            rows.append([
                name,
                spec.get("type", ""),
                "yes" if spec.get("required") else "no",
                str(spec.get("default", "")),
                spec.get("regex", ""),
            ])
        print_table(["Name", "Type", "Required", "Default", "Regex"], rows)
    if params.squid_params:
        print_info("\nSquid-level parameters:")
        rows = []
        for name, spec in params.squid_params.items():
            allowed = ", ".join(str(v) for v in spec["allowed"]) if isinstance(spec.get("allowed"), list) else ""
            rows.append([
                name,
                str(spec.get("default", "")),
                "yes" if spec.get("required") else "no",
                allowed[:50],
            ])
        print_table(["Name", "Default", "Required", "Allowed Values"], rows)
    if params.functions:
        print_info("\nOptional functions (extra credits):")
        rows = []
        for name, spec in params.functions.items():
            cpf = spec.get("credits_per_function", "")
            if isinstance(cpf, dict):
                cpf = cpf.get("current", cpf.get("legacy", ""))
            rows.append([name, str(cpf), str(spec.get("default", ""))])
        print_table(["Function", "Credits", "Default"], rows)


@crawlers_app.command("attrs")
def crawler_attributes(crawler: str = typer.Argument(..., help="Crawler slug, hash, or name")):
    """Show result attributes (columns) for a crawler."""
    from collections import OrderedDict
    from lobstr_cli.cli import get_client, _state
    from lobstr_cli.resolve import resolve_crawler
    client = get_client()
    all_crawlers = client.crawlers.list()
    crawler_id = resolve_crawler(crawler, all_crawlers)
    attrs = client.crawlers.attributes(crawler_id)
    if _state.get("json"):
        print_json([asdict(a) for a in attrs])
        return
    if not attrs:
        print_info("No attributes found for this crawler.")
        return
    groups: OrderedDict[str, list] = OrderedDict()
    for a in attrs:
        groups.setdefault(a.function, []).append(a)
    for func, items in groups.items():
        print_info(f"\n{func}:")
        rows = []
        for a in items:
            example = str(a.example) if a.example is not None else ""
            if len(example) > 40:
                example = example[:40] + "..."
            type_colors = {
                "string": "green", "text": "green",
                "integer": "cyan", "float": "cyan",
                "boolean": "yellow",
                "json": "magenta",
            }
            color = type_colors.get(a.type, "")
            if example and color:
                example = f"[{color}]{example}[/]"
            rows.append([
                f"[bold]{a.name}[/]",
                a.type,
                "[green]yes[/]" if a.is_main else "no",
                a.description,
                example,
            ])
        print_table(["Name", "Type", "Main", "Description", "Example"], rows)


@crawlers_app.command("search")
def search_crawlers(keyword: str = typer.Argument(..., help="Search keyword")):
    """Search crawlers by name."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    crawlers = client.crawlers.list()
    lower = keyword.lower()
    matches = [c for c in crawlers if lower in c.name.lower()]
    if _state.get("json"):
        print_json([asdict(c) for c in matches])
        return
    if not matches:
        print_info(f"No crawlers matching '{keyword}'")
        return
    rows = []
    for c in matches:
        status = ""
        if c.has_issues:
            status = "[yellow]issues[/]"
        elif not c.is_available:
            status = "[red]unavailable[/]"
        elif c.is_premium:
            status = "[cyan]premium[/]"
        rows.append([
            c.name,
            c.slug,
            c.id[:12],
            str(c.credits_per_row or "?"),
            str(c.max_concurrency),
            "yes" if c.account else "no",
            status,
        ])
    print_table(["Name", "Slug", "Hash", "Credits/Row", "Max Conc.", "Needs Account", "Status"], rows)



from __future__ import annotations

import time
from dataclasses import asdict
from pathlib import Path
from typing import Optional
import typer

from lobstr_cli.display import (
    print_json, print_success, print_info, print_error,
    make_progress, print_detail,
)

go_app = typer.Typer()


def _find_squid_by_name(client, name: str, crawler_id: str):
    """Find an existing squid by name and crawler."""
    items = client.squids.list(name=name)
    for s in items:
        if s.name == name and s.crawler == crawler_id:
            return s
    return None


@go_app.command("go")
def go(
    crawler: str = typer.Argument(..., help="Crawler name or hash"),
    inputs: Optional[list[str]] = typer.Argument(None, help="URLs or keywords for tasks"),
    file: Optional[Path] = typer.Option(None, "--file", "-f", help="File with inputs (one per line)"),
    key: str = typer.Option("url", "--key", "-k", help="Task param name (url, keyword, etc.)"),
    param: Optional[list[str]] = typer.Option(None, "--param", "-p", help="KEY=VALUE, repeatable"),
    concurrency: Optional[int] = typer.Option(None, "--concurrency", "-c"),
    output: str = typer.Option("results.csv", "--output", "-o", help="Output file path"),
    no_download: bool = typer.Option(False, "--no-download", help="Start run without waiting"),
    name: Optional[str] = typer.Option(None, "--name", help="Custom squid name"),
    delete: bool = typer.Option(False, "--delete", help="Delete squid after completion"),
    empty: bool = typer.Option(False, "--empty", help="Empty old tasks before adding new ones (when reusing squid)"),
):
    """Full workflow: create squid, add tasks, run, download."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()

    # 1. Resolve crawler
    print_info("Resolving crawler...")
    all_crawlers = client.crawlers.list()
    from lobstr_cli.resolve import resolve_crawler
    crawler_id = resolve_crawler(crawler, all_crawlers)
    crawler_name = next((c.name for c in all_crawlers if c.id == crawler_id), crawler_id[:12])
    print_info(f"Using crawler: {crawler_name}")

    # 2. Gather inputs
    task_inputs = list(inputs) if inputs else []
    if file:
        with open(file) as f:
            for line in f:
                line = line.strip()
                if line and not line.startswith("#"):
                    task_inputs.append(line)
    if not task_inputs:
        print_error("No inputs provided. Pass URLs/keywords as arguments or use --file.")
        raise typer.Exit(1)

    squid_id = None
    run_id = None
    created_new_squid = False
    try:
        # 3. Find existing or create squid
        existing = _find_squid_by_name(client, name, crawler_id) if name else None
        if existing:
            squid_id = existing.id
            print_info(f"Reusing squid: {existing.name} ({squid_id[:12]})")
            if empty:
                client.squids.empty(squid_id, type="url")
                print_info("Emptied old tasks")
        else:
            print_info(f"Creating squid ({len(task_inputs)} tasks)...")
            squid_obj = client.squids.create(crawler_id, name=name)
            squid_id = squid_obj.id
            created_new_squid = True
            print_info(f"Squid: {squid_obj.name} ({squid_id[:12]})")

        # 4. Update squid params if needed
        update_kwargs: dict = {}
        if concurrency is not None:
            update_kwargs["concurrency"] = concurrency
        if param:
            from lobstr_cli.resolve import parse_params
            update_kwargs["params"] = parse_params(param)
        if update_kwargs:
            client.squids.update(squid_id, **update_kwargs)

        # 5. Add tasks
        tasks = [{key: val} for val in task_inputs]
        result = client.tasks.add(squid=squid_id, tasks=tasks)
        print_info(f"Added {len(result.tasks)} tasks ({result.duplicated_count} duplicates)")

        # 6. Start run
        run = client.runs.start(squid=squid_id)
        run_id = run.id
        print_info(f"Run started: {run_id}")

        if no_download:
            if _state.get("json"):
                print_json({"squid": squid_id, "run": run_id})
            else:
                print_success(f"Run started. Watch with: lobstr run watch {run_id}")
            return

        # 7. Poll with progress
        with make_progress() as progress:
            ptask = progress.add_task("Running...", total=100)
            while True:
                stats = client.runs.stats(run_id)
                pct_str = stats.percent_done.replace("%", "")
                try:
                    pct = float(pct_str)
                except ValueError:
                    pct = 0
                progress.update(ptask, completed=pct, total=100,
                              description=f"Running... {stats.total_tasks_done}/{stats.total_tasks} tasks  ETA: {stats.eta or '?'}")
                if stats.is_done:
                    break
                time.sleep(3)

        # 8. Download (with retry — export may take a moment)
        final = client.runs.get(run_id)
        if _state.get("json"):
            print_json(asdict(final))
        else:
            downloaded = False
            for attempt in range(5):
                try:
                    client.runs.download(run_id, output)
                    downloaded = True
                    break
                except Exception:
                    pass
                if attempt == 0:
                    print_info("Waiting for export...")
                time.sleep(5)

            if downloaded:
                print_success(f"Downloaded {final.total_results} results to {output}")
            else:
                print_info("Export not ready. Try: lobstr run download " + run_id)

            print_detail([
                ("Status", final.status),
                ("Results", final.total_results),
                ("Duration", final.duration),
                ("Credits", final.credit_used),
            ])

        # 9. Delete squid if requested
        if delete:
            client.squids.delete(squid_id)
            print_info(f"Deleted squid {squid_id[:12]}")

    except KeyboardInterrupt:
        msg = "\nInterrupted."
        if squid_id:
            msg += f" Squid: {squid_id[:12]}"
        if run_id:
            msg += f" Run: {run_id}"
        print_info(msg)
        raise typer.Exit(0)
    except Exception:
        if created_new_squid and squid_id and not run_id:
            # Clean up orphaned squid if error before run started
            try:
                client.squids.delete(squid_id)
                print_info(f"Cleaned up squid {squid_id[:12]}")
            except Exception:
                print_info(f"Squid {squid_id[:12]} may need manual cleanup")
        raise





def test_get_config_dir(tmp_path, monkeypatch):
    monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path))
    from lobstr_cli.config import get_config_dir
    assert get_config_dir() == tmp_path / "lobstr"


def test_save_and_load_token(tmp_path, monkeypatch):
    monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path))
    from lobstr_cli.config import save_token, load_config
    save_token("test-token-123")
    cfg = load_config()
    assert cfg["auth"]["token"] == "test-token-123"


def test_load_config_missing_file(tmp_path, monkeypatch):
    monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path))
    from lobstr_cli.config import load_config
    cfg = load_config()
    assert cfg == {}


def test_get_token_from_env(monkeypatch):
    monkeypatch.setenv("LOBSTR_TOKEN", "env-token-456")
    from lobstr_cli.config import get_token
    assert get_token() == "env-token-456"


def test_get_token_from_config(tmp_path, monkeypatch):
    monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path))
    monkeypatch.delenv("LOBSTR_TOKEN", raising=False)
    from lobstr_cli.config import save_token, get_token
    save_token("config-token-789")
    assert get_token() == "config-token-789"


def test_save_alias(tmp_path, monkeypatch):
    monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path))
    from lobstr_cli.config import save_token, save_alias, load_config
    save_token("tok")
    save_alias("reviews", "abc123hash")
    cfg = load_config()
    assert cfg["aliases"]["reviews"] == "abc123hash"


def test_resolve_alias(tmp_path, monkeypatch):
    monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path))
    from lobstr_cli.config import save_token, save_alias, resolve_alias
    save_token("tok")
    save_alias("reviews", "abc123hash")
    assert resolve_alias("@reviews") == "abc123hash"
    assert resolve_alias("not-an-alias") == "not-an-alias"



import sys

from lobstr_cli.cli import app
from lobstrio.exceptions import APIError
from lobstr_cli.display import print_error


def main():
    try:
        app()
    except APIError as e:
        print_error(str(e))
        sys.exit(1)


if __name__ == "__main__":
    main()



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.crawler import Crawler
from lobstrio.models.squid import Squid
from lobstrio.models.task import Task, AddTasksResult
from lobstrio.models.run import Run, RunStats


runner = CliRunner()

CRAWLERS = [
    Crawler(
        id="crawler1abc", name="Google Maps Leads Scraper",
        slug="google-maps-leads-scraper", description=None,
        credits_per_row=3, credits_per_email=None, max_concurrency=5,
        account=False, has_email_verification=False, is_public=True,
        is_premium=False, is_available=True, has_issues=False, rank=1,
    ),
]


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client(squids_by_name=None):
    """Create a mock client for go command tests."""
    mock = MagicMock()
    mock.crawlers.list.return_value = CRAWLERS
    mock.squids.list.return_value = squids_by_name or []
    mock.squids.create.return_value = Squid(
        id="newsquid123", name="Test Squid", crawler="crawler1abc",
        crawler_name="Google Maps Leads Scraper", is_active=True, is_ready=False,
        concurrency=1, to_complete=None, last_run_status=None,
        last_run_at=None, total_runs=0, export_unique_results=False,
        params={},
    )
    mock.squids.update.return_value = None
    mock.squids.empty.return_value = {"deleted_count": 0}
    mock.squids.delete.return_value = {}

    def _make_add_result(*, squid, tasks):
        return AddTasksResult(
            tasks=[Task(id=f"t{i}", is_active=True, params=t, status=None, created_at=None)
                   for i, t in enumerate(tasks)],
            duplicated_count=0,
        )
    mock.tasks.add.side_effect = _make_add_result

    mock.runs.start.return_value = Run(
        id="run123", status="running", total_results=0,
        total_unique_results=0, duration=0, credit_used=0,
        origin="api", done_reason=None, done_reason_desc=None,
        export_done=False, started_at=None, ended_at=None,
    )
    mock.runs.stats.return_value = RunStats(
        percent_done="100%", total_tasks=1, total_tasks_done=1,
        total_tasks_left=0, total_results=10, duration=30,
        eta="0s", current_task=None, is_done=True,
    )
    mock.runs.get.return_value = Run(
        id="run123", status="finished", total_results=10,
        total_unique_results=10, duration=30, credit_used=5,
        origin="api", done_reason="completed", done_reason_desc=None,
        export_done=True, started_at=None, ended_at=None,
    )
    mock.runs.download.return_value = None
    return mock


class TestGoBasic:
    def test_go_full_workflow(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://maps.google.com/place1"])
        assert result.exit_code == 0
        assert "Downloaded" in result.output

    def test_go_multiple_inputs(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com", "https://b.com"])
        assert result.exit_code == 0

    def test_go_no_inputs_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps"])
        assert result.exit_code == 1


class TestGoKey:
    def test_custom_key(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "pizza", "--key", "keyword"])
        assert result.exit_code == 0
        mock.tasks.add.assert_called_once_with(squid="newsquid123", tasks=[{"keyword": "pizza"}])


class TestGoFile:
    def test_file_input(self, tmp_path):
        f = tmp_path / "urls.txt"
        f.write_text("https://a.com\nhttps://b.com\n# comment\n\n")
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "--file", str(f)])
        assert result.exit_code == 0
        call_args = mock.tasks.add.call_args
        tasks = call_args[1]["tasks"]
        assert len(tasks) == 2  # comment and empty line filtered


class TestGoNoDownload:
    def test_no_download(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com", "--no-download"])
        assert result.exit_code == 0
        mock.runs.download.assert_not_called()


class TestGoDelete:
    def test_delete_after_completion(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com", "--delete"])
        assert result.exit_code == 0
        mock.squids.delete.assert_called_once()


class TestGoReuse:
    def test_reuse_existing_squid(self):
        existing = [
            Squid(
                id="existing123", name="MyScraper", crawler="crawler1abc",
                crawler_name="Google Maps", is_active=True, is_ready=True,
                concurrency=1, to_complete=None, last_run_status=None,
                last_run_at=None, total_runs=0, export_unique_results=False,
                params={},
            ),
        ]
        mock = _mock_client(squids_by_name=existing)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com", "--name", "MyScraper"])
        assert result.exit_code == 0
        # Should NOT have created a new squid
        mock.squids.create.assert_not_called()

    def test_reuse_with_empty(self):
        existing = [
            Squid(
                id="existing123", name="MyScraper", crawler="crawler1abc",
                crawler_name="Google Maps", is_active=True, is_ready=True,
                concurrency=1, to_complete=None, last_run_status=None,
                last_run_at=None, total_runs=0, export_unique_results=False,
                params={},
            ),
        ]
        mock = _mock_client(squids_by_name=existing)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com", "--name", "MyScraper", "--empty"])
        assert result.exit_code == 0
        mock.squids.empty.assert_called_once()


class TestGoParams:
    def test_squid_params(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com",
                                         "--param", "max_results=200", "--param", "language=English"])
        assert result.exit_code == 0
        mock.squids.update.assert_called_once()

    def test_concurrency(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com", "--concurrency", "3"])
        assert result.exit_code == 0


class TestGoCleanup:
    def test_cleanup_orphaned_squid_on_error(self):
        mock = _mock_client()
        mock.tasks.add.side_effect = Exception("Task creation failed")
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["go", "Google Maps", "https://a.com"])
        assert result.exit_code != 0
        mock.squids.delete.assert_called_once()



from __future__ import annotations

import json as json_mod
from dataclasses import asdict
from pathlib import Path
from typing import Optional
import typer

from lobstr_cli.display import print_json, print_table, print_detail, print_success, print_error, print_info
from lobstr_cli.resolve import resolve_account as _resolve_account

accounts_app = typer.Typer(no_args_is_help=True)


@accounts_app.command("ls")
def list_accounts():
    """List your accounts."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    items = client.accounts.list()
    if _state.get("json"):
        print_json([asdict(a) for a in items])
        return
    rows = []
    for a in items:
        rows.append([
            a.username,
            a.id[:12],
            a.type,
            a.status_code_info or "",
            (a.last_synchronization_time or "")[:10] or "—",
        ])
    print_table(["Username", "Hash", "Type", "Status", "Last Sync"], rows)


@accounts_app.command("show")
def show_account(account: str = typer.Argument(..., help="Account hash or username")):
    """Show account details."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    account_id = _resolve_account(client, account)
    data = client.accounts.get(account_id)
    if _state.get("json"):
        print_json(asdict(data))
        return
    squid_names = ", ".join(s.get("name", "") for s in data.squids) if data.squids else "—"
    print_detail([
        ("Username", data.username),
        ("Hash", data.id),
        ("Type", data.type),
        ("Status", data.status_code_info),
        ("Status Detail", data.status_code_description),
        ("Base URL", data.baseurl),
        ("Created", data.created_at),
        ("Last Sync", data.last_synchronization_time),
        ("Updated", data.updated_at),
        ("Squids", squid_names),
    ])


@accounts_app.command("rm")
def delete_account(
    account: str = typer.Argument(..., help="Account hash or username"),
    force: bool = typer.Option(False, "--force", "-f", help="Skip confirmation"),
):
    """Delete an account permanently."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    account_id = _resolve_account(client, account)
    if not force:
        typer.confirm(f"Delete account {account_id[:12]}? This is permanent.", abort=True)
    result = client.accounts.delete(account_id)
    if _state.get("json"):
        print_json(result)
        return
    print_success(f"Deleted account {account_id[:12]}")


@accounts_app.command("types")
def list_account_types():
    """List available account types."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    items = client.accounts.types()
    if _state.get("json"):
        print_json([asdict(t) for t in items])
        return
    rows = []
    for t in items:
        cookie_names = ", ".join(c.get("name", "") for c in t.cookies) if t.cookies else "all session cookies"
        rows.append([
            t.name,
            t.domain,
            t.baseurl,
            cookie_names[:50],
        ])
    print_table(["Type", "Domain", "Base URL", "Required Cookies"], rows)


def _parse_cookies(
    cookie: list[str] | None,
    cookies_json: str | None,
    cookies_file: Path | None,
) -> dict:
    """Parse cookies from any of the three input methods."""
    result = {}
    if cookie:
        from lobstr_cli.resolve import parse_params
        result.update(parse_params(cookie))
    if cookies_json:
        result.update(json_mod.loads(cookies_json))
    if cookies_file:
        result.update(json_mod.loads(cookies_file.read_text()))
    if not result:
        print_error("No cookies provided. Use --cookie, --cookies-json, or --cookies-file")
        raise typer.Exit(1)
    return result


@accounts_app.command("sync")
def sync_account(
    type: str = typer.Argument(..., help="Account type (e.g. twitter-sync, facebook-sync)"),
    account: Optional[str] = typer.Option(None, "--account", help="Existing account hash to refresh"),
    cookie: Optional[list[str]] = typer.Option(None, "--cookie", "-c", help="Cookie KEY=VALUE, repeatable"),
    cookies_json: Optional[str] = typer.Option(None, "--cookies-json", help="Cookies as JSON string"),
    cookies_file: Optional[Path] = typer.Option(None, "--cookies-file", help="Path to cookies JSON file", exists=True),
):
    """Sync a new account or refresh cookies for an existing one."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    cookies = _parse_cookies(cookie, cookies_json, cookies_file)
    result = client.accounts.sync(type, cookies, account=account)
    if _state.get("json"):
        print_json(result)
        return
    sync_id = result.get("id", "")
    print_success(f"Sync started: {sync_id} (status: {result.get('status_text', '?')})")
    print_info(f"Check status with: lobstr accounts sync-status {sync_id}")


@accounts_app.command("sync-status")
def sync_status(sync_id: str = typer.Argument(..., help="Sync task ID")):
    """Check account sync status."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    result = client.accounts.sync_status(sync_id)
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_detail([
        ("Sync ID", result.id),
        ("Status Code", result.status_code),
        ("Status", result.status_text),
        ("Account Hash", result.account_hash),
    ])


@accounts_app.command("update")
def update_account(
    account: str = typer.Argument(..., help="Account hash or username"),
    type: str = typer.Option(..., "--type", help="Account type (e.g. sales-nav-sync)"),
    param: Optional[list[str]] = typer.Option(None, "--param", help="Limit KEY=VALUE, repeatable"),
):
    """Update account limits."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    account_id = _resolve_account(client, account)
    if not param:
        print_error("No params specified. Use --param to set limits (e.g. --param batch=20)")
        raise typer.Exit(1)
    from lobstr_cli.resolve import parse_params
    result = client.accounts.update(account_id, type=type, params=parse_params(param))
    if _state.get("json"):
        print_json(result)
        return
    print_success(f"Updated limits for account {account_id[:12]}")



from __future__ import annotations

from typing import Any

from lobstr_cli.display import print_error


def require_full_hash(value: str, label: str = "resource") -> None:
    """Raise a clear error if a partial hash is given for run/task endpoints."""
    if len(value) < 32:
        print_error(
            f"The API requires a full 32-character hash for {label}s. "
            f"Got {len(value)} characters: '{value}'. "
            f"Use `lobstr run ls` or `lobstr task ls` to get full hashes."
        )
        raise SystemExit(1)


def match_hash_prefix(prefix: str, items: list[Any], key: str = "id") -> str:
    """Match by hash prefix. Items can be dicts or model objects."""
    def _get(item: Any, k: str) -> str:
        return item[k] if isinstance(item, dict) else getattr(item, k)

    for item in items:
        if _get(item, key) == prefix:
            return prefix
    matches = [_get(item, key) for item in items if _get(item, key).startswith(prefix)]
    if len(matches) == 1:
        return matches[0]
    if len(matches) == 0:
        print_error(f"No match for prefix '{prefix}'")
        raise SystemExit(1)
    print_error(f"Ambiguous prefix '{prefix}' matches: {', '.join(matches[:5])}")
    raise SystemExit(1)


def _attr(item: Any, name: str, default: str = "") -> str:
    """Get attribute from model object or dict."""
    if isinstance(item, dict):
        return item.get(name, default)
    return getattr(item, name, default)


def match_slug(slug: str, items: list[Any], label: str = "squid") -> str:
    lower = slug.lower()
    for item in items:
        if _attr(item, "slug").lower() == lower:
            return _attr(item, "id")
    matches = [item for item in items if _attr(item, "slug").lower().startswith(lower)]
    if len(matches) == 1:
        return _attr(matches[0], "id")
    if len(matches) == 0:
        print_error(f"No {label} matching slug '{slug}'")
        raise SystemExit(1)
    names = [_attr(m, "slug") or _attr(m, "name") for m in matches[:5]]
    print_error(f"Ambiguous {label} slug '{slug}' matches: {', '.join(names)}")
    raise SystemExit(1)


def match_name(name: str, items: list[Any], label: str = "squid") -> str:
    lower = name.lower()
    for item in items:
        if _attr(item, "name").lower() == lower:
            return _attr(item, "id")
    matches = [item for item in items if lower in _attr(item, "name").lower()]
    if len(matches) == 1:
        return _attr(matches[0], "id")
    if len(matches) == 0:
        print_error(f"No {label} matching '{name}'")
        raise SystemExit(1)
    names = [_attr(m, "name") for m in matches[:5]]
    print_error(f"Ambiguous {label} name '{name}' matches: {', '.join(names)}")
    raise SystemExit(1)


def resolve_squid(client, identifier: str) -> str:
    from lobstr_cli.config import resolve_alias
    identifier = resolve_alias(identifier)
    items = client.squids.list()
    if all(c in "0123456789abcdef" for c in identifier.lower()):
        try:
            return match_hash_prefix(identifier.lower(), items)
        except SystemExit:
            pass
    return match_name(identifier, items, "squid")


def match_username(username: str, items: list[Any], label: str = "account") -> str:
    lower = username.lower()
    for item in items:
        if _attr(item, "username").lower() == lower:
            return _attr(item, "id")
    matches = [item for item in items if lower in _attr(item, "username").lower()]
    if len(matches) == 1:
        return _attr(matches[0], "id")
    if len(matches) == 0:
        print_error(f"No {label} matching username '{username}'")
        raise SystemExit(1)
    names = [_attr(m, "username") for m in matches[:5]]
    print_error(f"Ambiguous {label} username '{username}' matches: {', '.join(names)}")
    raise SystemExit(1)


def resolve_account(client, identifier: str) -> str:
    items = client.accounts.list()
    if all(c in "0123456789abcdef" for c in identifier.lower()):
        try:
            return match_hash_prefix(identifier.lower(), items)
        except SystemExit:
            pass
    return match_username(identifier, items, "account")


def match_crawler_name(name: str, crawlers: list[Any]) -> str:
    return match_name(name, crawlers, "crawler")


def resolve_crawler(identifier: str, crawlers: list[Any]) -> str:
    if all(c in "0123456789abcdef" for c in identifier.lower()):
        try:
            return match_hash_prefix(identifier.lower(), crawlers)
        except SystemExit:
            pass
    if "-" in identifier:
        return match_slug(identifier, crawlers, "crawler")
    return match_crawler_name(identifier, crawlers)


def parse_param_value(value: str):
    if value.lower() in ("true", "false"):
        return value.lower() == "true"
    if value.lower() == "none":
        return None
    try:
        return int(value)
    except ValueError:
        pass
    try:
        return float(value)
    except ValueError:
        pass
    return value


def parse_params(param_list: list[str]) -> dict:
    params = {}
    for p in param_list:
        k, _, v = p.partition("=")
        params[k] = parse_param_value(v)
    return params



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.crawler import Crawler
from lobstrio.models.squid import Squid


runner = CliRunner()

CRAWLERS = [
    Crawler(
        id="crawler1", name="Google Maps Leads Scraper",
        slug="google-maps-leads-scraper", description=None,
        credits_per_row=3, credits_per_email=None, max_concurrency=5,
        account=False, has_email_verification=False, is_public=True,
        is_premium=False, is_available=True, has_issues=False, rank=1,
    ),
]

SQUIDS = [
    Squid(
        id="squid1abc123def456", name="My Squid", crawler="crawler1",
        crawler_name="Google Maps", is_active=True, is_ready=True,
        concurrency=3, to_complete=0, last_run_status="finished",
        last_run_at="2025-01-01", total_runs=5, export_unique_results=True,
        params={"max_results": 100},
    ),
]


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client():
    mock = MagicMock()
    mock.squids.list.return_value = SQUIDS
    mock.crawlers.list.return_value = CRAWLERS
    return mock


class TestSquidCreate:
    def test_create_squid(self):
        mock = _mock_client()
        mock.squids.create.return_value = Squid(
            id="newsquid123", name="My New Squid", crawler="crawler1",
            crawler_name="Google Maps", is_active=True, is_ready=False,
            concurrency=1, to_complete=None, last_run_status=None,
            last_run_at=None, total_runs=0, export_unique_results=False,
            params={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "create", "Google Maps"])
        assert result.exit_code == 0
        assert "Created" in result.output

    def test_create_with_name(self):
        mock = _mock_client()
        mock.squids.create.return_value = Squid(
            id="newsquid123", name="Custom Name", crawler="crawler1",
            crawler_name="Google Maps", is_active=True, is_ready=False,
            concurrency=1, to_complete=None, last_run_status=None,
            last_run_at=None, total_runs=0, export_unique_results=False,
            params={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "create", "Google Maps", "--name", "Custom Name"])
        assert result.exit_code == 0
        mock.squids.create.assert_called_once_with("crawler1", name="Custom Name")

    def test_create_json_mode(self):
        mock = _mock_client()
        mock.squids.create.return_value = Squid(
            id="newsquid123", name="Squid", crawler="crawler1",
            crawler_name="Google Maps", is_active=True, is_ready=False,
            concurrency=1, to_complete=None, last_run_status=None,
            last_run_at=None, total_runs=0, export_unique_results=False,
            params={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "squid", "create", "Google Maps"])
        assert result.exit_code == 0
        assert "newsquid123" in result.output

    def test_create_by_crawler_slug(self):
        mock = _mock_client()
        mock.squids.create.return_value = Squid(
            id="newsquid123", name="New Squid", crawler="crawler1",
            crawler_name="Google Maps", is_active=True, is_ready=False,
            concurrency=1, to_complete=None, last_run_status=None,
            last_run_at=None, total_runs=0, export_unique_results=False,
            params={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "create", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "Created" in result.output
        mock.squids.create.assert_called_once_with("crawler1", name=None)


class TestSquidLs:
    def test_list_squids(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "ls"])
        assert result.exit_code == 0
        assert "My Squid" in result.output

    def test_list_with_pagination(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "ls", "--limit", "10", "--page", "2"])
        assert result.exit_code == 0
        mock.squids.list.assert_called_once_with(limit=10, page=2, name=None)

    def test_list_json(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "squid", "ls"])
        assert result.exit_code == 0


class TestSquidShow:
    SQUID_DETAIL = Squid(
        id="squid1abc123def456", name="My Squid", crawler="crawler1",
        crawler_name="Google Maps", is_active=True, is_ready=True,
        concurrency=3, to_complete=0, last_run_status="finished",
        last_run_at="2025-01-01", total_runs=5, export_unique_results=True,
        params={"max_results": 100},
    )

    def test_show_by_name(self):
        mock = _mock_client()
        mock.squids.get.return_value = self.SQUID_DETAIL
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "show", "My Squid"])
        assert result.exit_code == 0
        assert "My Squid" in result.output

    def test_show_by_name_substring(self):
        mock = _mock_client()
        mock.squids.get.return_value = self.SQUID_DETAIL
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "show", "Squid"])
        assert result.exit_code == 0

    def test_show_by_hash_prefix(self):
        squids = [
            Squid(
                id="aabb11cc22dd33ee44ff5566", name="My Squid", crawler="c1",
                crawler_name="Google Maps", is_active=True, is_ready=True,
                concurrency=1, to_complete=None, last_run_status=None,
                last_run_at=None, total_runs=0, export_unique_results=False,
                params={},
            ),
        ]
        mock = MagicMock()
        mock.squids.list.return_value = squids
        mock.squids.get.return_value = squids[0]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "show", "aabb11"])
        assert result.exit_code == 0


class TestSquidUpdate:
    def test_update_concurrency(self):
        mock = _mock_client()
        mock.squids.update.return_value = SQUIDS[0]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "update", "My Squid", "--concurrency", "5"])
        assert result.exit_code == 0
        mock.squids.update.assert_called_once()
        call_kwargs = mock.squids.update.call_args
        assert call_kwargs[1]["concurrency"] == 5

    def test_update_with_params(self):
        mock = _mock_client()
        mock.squids.update.return_value = SQUIDS[0]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "update", "My Squid", "--param", "max_results=200"])
        assert result.exit_code == 0
        call_kwargs = mock.squids.update.call_args
        assert call_kwargs[1]["params"]["max_results"] == 200

    def test_update_no_options_error(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "update", "My Squid"])
        assert result.exit_code == 1


class TestSquidEmpty:
    def test_empty_squid(self):
        mock = _mock_client()
        mock.squids.empty.return_value = {"deleted_count": 10}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "empty", "My Squid"])
        assert result.exit_code == 0
        assert "10" in result.output


class TestSquidRm:
    def test_delete_with_force(self):
        mock = _mock_client()
        mock.squids.delete.return_value = {"id": "squid1abc123def456", "deleted": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "rm", "My Squid", "--force"])
        assert result.exit_code == 0
        assert "Deleted" in result.output

    def test_delete_without_force_prompts(self):
        mock = _mock_client()
        mock.squids.delete.return_value = {"id": "squid1abc123def456", "deleted": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "rm", "My Squid"], input="y\n")
        assert result.exit_code == 0

    def test_delete_aborted(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["squid", "rm", "My Squid"], input="n\n")
        assert result.exit_code != 0



from __future__ import annotations

from dataclasses import asdict

import typer

from lobstr_cli.config import save_token, save_alias, load_config, get_config_path
from lobstr_cli.display import print_success, print_detail, print_json

config_app = typer.Typer(no_args_is_help=True)
whoami_app = typer.Typer()


@config_app.command("set-token")
def set_token(token: str = typer.Argument(..., help="Your Lobstr API token (get it at https://app.lobstr.io/dashboard/api)")):
    """Store your API token. Get your token at https://app.lobstr.io/dashboard/api"""
    save_token(token)
    print_success(f"Token saved to {get_config_path()}")


@config_app.command("show")
def show_config():
    """Show current configuration."""
    from lobstr_cli.cli import _state
    cfg = load_config()
    if _state.get("json"):
        masked = dict(cfg)
        if "auth" in masked and "token" in masked["auth"]:
            t = masked["auth"]["token"]
            masked["auth"]["token"] = t[:8] + "..." if len(t) > 8 else "***"
        print_json(masked)
        return
    token = cfg.get("auth", {}).get("token", "")
    masked = token[:8] + "..." if len(token) > 8 else "(not set)"
    fields = [
        ("Config file", str(get_config_path())),
        ("Token", masked),
    ]
    defaults = cfg.get("defaults", {})
    for k, v in defaults.items():
        fields.append((f"Default {k}", v))
    aliases = cfg.get("aliases", {})
    if aliases:
        for name, h in aliases.items():
            fields.append((f"Alias @{name}", h))
    print_detail(fields)


@config_app.command("set-alias")
def set_alias(
    name: str = typer.Argument(..., help="Alias name (used as @name)"),
    squid_id: str = typer.Argument(..., help="Squid hash to alias"),
):
    """Create an alias for a squid hash."""
    save_alias(name, squid_id)
    print_success(f"Alias @{name} -> {squid_id[:12]}")


@whoami_app.command("whoami")
def whoami():
    """Show current user and balance."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    me = client.me()
    balance = client.balance()
    if _state.get("json"):
        print_json({**asdict(me), "balance": asdict(balance)})
        return
    name = f"{me.first_name} {me.last_name}".strip() or None
    plan_name = me.plan[0].get("name") if me.plan else None
    plan_status = me.plan[0].get("status") if me.plan else None
    print_detail([
        ("Name", name),
        ("Email", me.email),
        ("Plan", plan_name),
        ("Status", plan_status),
        ("Staff", me.is_staff),
        ("Balance", f"{balance.available} credits"),
        ("Consumed", balance.consumed),
        ("Slots Used", f"{balance.used_slots}/{balance.total_available_slots}"),
    ])



from __future__ import annotations

from dataclasses import asdict
from pathlib import Path
import typer

from lobstr_cli.display import print_json, print_table, print_detail, print_success, print_info
from lobstr_cli.resolve import resolve_squid as _resolve_squid, require_full_hash

task_app = typer.Typer(no_args_is_help=True)


@task_app.command("add")
def add_tasks(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    values: list[str] = typer.Argument(..., help="URLs or keywords to add as tasks"),
    key: str = typer.Option("url", "--key", "-k", help="Task param name (url, keyword, etc.)"),
):
    """Add tasks to a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    tasks = [{key: v} for v in values]
    result = client.tasks.add(squid=squid_id, tasks=tasks)
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"Added {len(result.tasks)} tasks ({result.duplicated_count} duplicates skipped)")


@task_app.command("upload")
def upload_tasks(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    file: Path = typer.Argument(..., help="CSV/TSV file path", exists=True),
):
    """Upload tasks from a CSV/TSV file."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.tasks.upload(squid=squid_id, file=file)
    if _state.get("json"):
        print_json(result)
        return
    upload_id = result.get("id", "")
    print_success(f"Upload started: {upload_id}")
    print_info(f"Check status with: lobstr task upload-status {upload_id}")


@task_app.command("upload-status")
def upload_status(upload_id: str = typer.Argument(..., help="Upload task ID")):
    """Check upload progress."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    result = client.tasks.upload_status(upload_id)
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_detail([
        ("State", result.state),
        ("Valid", result.meta.valid),
        ("Inserted", result.meta.inserted),
        ("Duplicates", result.meta.duplicates),
        ("Invalid", result.meta.invalid),
    ])


@task_app.command("ls")
def list_tasks(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    limit: int = typer.Option(50, "--limit"),
    page: int = typer.Option(1, "--page"),
):
    """List tasks for a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    items = client.tasks.list(squid=squid_id, limit=limit, page=page)
    if _state.get("json"):
        print_json([asdict(t) for t in items])
        return
    rows = []
    for t in items:
        url = t.params.get("url", str(t.params)[:60])
        rows.append([
            t.id,
            "yes" if t.is_active else "no",
            str(url)[:70],
            (t.created_at or "")[:10],
        ])
    print_table(["Hash", "Active", "URL/Params", "Created"], rows)


@task_app.command("show")
def show_task(task_id: str = typer.Argument(..., help="Full task hash")):
    """Show task details."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(task_id, "task")
    data = client.tasks.get(task_id)
    if _state.get("json"):
        print_json(asdict(data))
        return
    fields = [
        ("Hash", data.id),
        ("Active", data.is_active),
    ]
    if data.status:
        fields.extend([
            ("Status", data.status.status),
            ("Results", data.status.total_results),
            ("Pages", data.status.total_pages),
            ("Done Reason", data.status.done_reason),
            ("Errors", data.status.has_errors),
        ])
    fields.append(("Params", data.params))
    print_detail(fields)


@task_app.command("rm")
def delete_task(task_id: str = typer.Argument(..., help="Full task hash")):
    """Delete a task."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(task_id, "task")
    result = client.tasks.delete(task_id)
    if _state.get("json"):
        print_json(result)
        return
    print_success(f"Deleted task {task_id}")



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.squid import Squid
from lobstrio.models.delivery import (
    EmailDelivery, GoogleSheetDelivery, S3Delivery,
    WebhookDelivery, WebhookEvents, SFTPDelivery,
)


runner = CliRunner()

SQUIDS = [
    Squid(
        id="squid1abc123def456", name="My Squid", crawler="c1",
        crawler_name="Google Maps", is_active=True, is_ready=True,
        concurrency=1, to_complete=None, last_run_status=None,
        last_run_at=None, total_runs=0, export_unique_results=False,
        params={},
    ),
]


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client():
    mock = MagicMock()
    mock.squids.list.return_value = SQUIDS
    return mock


class TestDeliveryEmail:
    def test_configure_email(self):
        mock = _mock_client()
        mock.delivery.email.return_value = EmailDelivery(email="user@example.com", notifications=True)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "email", "My Squid", "--email", "user@example.com"])
        assert result.exit_code == 0
        mock.delivery.email.assert_called_once_with(
            "squid1abc123def456", email="user@example.com", notifications=True,
        )

    def test_configure_email_no_notifications(self):
        mock = _mock_client()
        mock.delivery.email.return_value = EmailDelivery(email="user@example.com", notifications=False)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "email", "My Squid", "--email", "user@example.com", "--no-notifications"])
        assert result.exit_code == 0
        mock.delivery.email.assert_called_once_with(
            "squid1abc123def456", email="user@example.com", notifications=False,
        )

    def test_email_json_mode(self):
        mock = _mock_client()
        mock.delivery.email.return_value = EmailDelivery(email="user@example.com", notifications=True)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "delivery", "email", "My Squid", "--email", "user@example.com"])
        assert result.exit_code == 0


class TestDeliveryGoogleSheet:
    def test_configure_googlesheet(self):
        mock = _mock_client()
        mock.delivery.google_sheet.return_value = GoogleSheetDelivery(
            url="https://docs.google.com/spreadsheets/d/123", append=False, is_active=True,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "googlesheet", "My Squid", "--url", "https://docs.google.com/spreadsheets/d/123"])
        assert result.exit_code == 0
        assert "configured" in result.output

    def test_configure_with_append(self):
        mock = _mock_client()
        mock.delivery.google_sheet.return_value = GoogleSheetDelivery(
            url="https://...", append=True, is_active=True,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "googlesheet", "My Squid", "--url", "https://...", "--append"])
        assert result.exit_code == 0
        mock.delivery.google_sheet.assert_called_once_with(
            "squid1abc123def456", url="https://...", append=True, is_active=True,
        )


class TestDeliveryS3:
    def test_configure_s3(self):
        mock = _mock_client()
        mock.delivery.s3.return_value = S3Delivery(bucket="mybucket", target_path="exports", is_active=True)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "s3", "My Squid", "--bucket", "mybucket", "--target-path", "exports"])
        assert result.exit_code == 0
        assert "configured" in result.output

    def test_s3_with_credentials(self):
        mock = _mock_client()
        mock.delivery.s3.return_value = S3Delivery(bucket="b", target_path="p", is_active=True)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, [
                "delivery", "s3", "My Squid",
                "--bucket", "b", "--target-path", "p",
                "--aws-access-key", "AKIA...", "--aws-secret-key", "secret",
            ])
        assert result.exit_code == 0
        mock.delivery.s3.assert_called_once_with(
            "squid1abc123def456", bucket="b", target_path="p",
            aws_access_key="AKIA...", aws_secret_key="secret", is_active=True,
        )


class TestDeliveryWebhook:
    def test_configure_webhook(self):
        mock = _mock_client()
        mock.delivery.webhook.return_value = WebhookDelivery(
            url="https://hook.example.com", is_active=True, retry=True,
            events=WebhookEvents(run_running=False, run_paused=False, run_done=True, run_error=True),
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "webhook", "My Squid", "--url", "https://hook.example.com"])
        assert result.exit_code == 0
        assert "configured" in result.output
        mock.delivery.webhook.assert_called_once_with(
            "squid1abc123def456", url="https://hook.example.com",
            is_active=True, retry=True,
            on_running=False, on_paused=False, on_done=True, on_error=True,
        )


class TestDeliverySftp:
    def test_configure_sftp(self):
        mock = _mock_client()
        mock.delivery.sftp.return_value = SFTPDelivery(
            host="sftp.example.com", port=22, username="user",
            directory="/upload", is_active=True,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, [
                "delivery", "sftp", "My Squid",
                "--host", "sftp.example.com",
                "--username", "user", "--password", "pass",
                "--directory", "/upload",
            ])
        assert result.exit_code == 0
        assert "configured" in result.output


class TestDeliveryTests:
    def test_test_email_pass(self):
        mock = _mock_client()
        mock.delivery.test_email.return_value = {"success": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "test-email", "--email", "user@example.com"])
        assert result.exit_code == 0
        assert "passed" in result.output

    def test_test_email_fail(self):
        mock = _mock_client()
        mock.delivery.test_email.return_value = {"success": False}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "test-email", "--email", "bad@example.com"])
        assert result.exit_code == 1

    def test_test_googlesheet(self):
        mock = _mock_client()
        mock.delivery.test_google_sheet.return_value = {"success": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "test-googlesheet", "--url", "https://docs.google.com/..."])
        assert result.exit_code == 0

    def test_test_s3(self):
        mock = _mock_client()
        mock.delivery.test_s3.return_value = {"success": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "test-s3", "--bucket", "mybucket"])
        assert result.exit_code == 0

    def test_test_webhook(self):
        mock = _mock_client()
        mock.delivery.test_webhook.return_value = {"success": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["delivery", "test-webhook", "--url", "https://hook.example.com"])
        assert result.exit_code == 0

    def test_test_sftp(self):
        mock = _mock_client()
        mock.delivery.test_sftp.return_value = {"success": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, [
                "delivery", "test-sftp",
                "--host", "sftp.example.com",
                "--username", "user", "--password", "pass",
                "--directory", "/upload",
            ])
        assert result.exit_code == 0

    def test_test_json_mode(self):
        mock = _mock_client()
        mock.delivery.test_email.return_value = {"success": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "delivery", "test-email", "--email", "user@example.com"])
        assert result.exit_code == 0



from __future__ import annotations

from dataclasses import asdict
from typing import Optional
import typer

from lobstr_cli.display import print_json, print_detail, print_success, print_error
from lobstr_cli.resolve import resolve_squid as _resolve_squid

delivery_app = typer.Typer(no_args_is_help=True)


@delivery_app.command("email")
def configure_email(
    squid: str = typer.Argument(..., help="Squid hash, name, or alias"),
    email: str = typer.Option(..., "--email", help="Email address"),
    notifications: bool = typer.Option(True, "--notifications/--no-notifications", help="Send notifications on completion"),
):
    """Configure email delivery for a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.delivery.email(squid_id, email=email, notifications=notifications)
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"Email delivery configured for {squid_id[:12]}")
    print_detail([
        ("Email", result.email),
        ("Notifications", result.notifications),
    ])


@delivery_app.command("googlesheet")
def configure_googlesheet(
    squid: str = typer.Argument(..., help="Squid hash, name, or alias"),
    url: str = typer.Option(..., "--url", help="Public Google Sheet URL"),
    append: bool = typer.Option(False, "--append/--no-append", help="Append results vs overwrite"),
    active: bool = typer.Option(True, "--active/--inactive", help="Enable/disable export"),
):
    """Configure Google Sheet delivery for a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.delivery.google_sheet(squid_id, url=url, append=append, is_active=active)
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"Google Sheet delivery configured for {squid_id[:12]}")
    print_detail([
        ("URL", result.url),
        ("Append", result.append),
        ("Active", result.is_active),
    ])


@delivery_app.command("s3")
def configure_s3(
    squid: str = typer.Argument(..., help="Squid hash, name, or alias"),
    bucket: str = typer.Option(..., "--bucket", help="S3 bucket name"),
    target_path: str = typer.Option(..., "--target-path", help="Folder path inside bucket"),
    aws_access_key: Optional[str] = typer.Option(None, "--aws-access-key", help="AWS Access Key"),
    aws_secret_key: Optional[str] = typer.Option(None, "--aws-secret-key", help="AWS Secret Key"),
    active: bool = typer.Option(True, "--active/--inactive", help="Enable/disable export"),
):
    """Configure S3 delivery for a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.delivery.s3(
        squid_id, bucket=bucket, target_path=target_path,
        aws_access_key=aws_access_key, aws_secret_key=aws_secret_key, is_active=active,
    )
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"S3 delivery configured for {squid_id[:12]}")
    print_detail([
        ("Bucket", result.bucket),
        ("Target Path", result.target_path),
        ("Active", result.is_active),
    ])


@delivery_app.command("webhook")
def configure_webhook(
    squid: str = typer.Argument(..., help="Squid hash, name, or alias"),
    url: str = typer.Option(..., "--url", help="Webhook endpoint URL"),
    retry: bool = typer.Option(True, "--retry/--no-retry", help="Retry failed deliveries"),
    on_running: bool = typer.Option(False, "--on-running", help="Subscribe to run start events"),
    on_paused: bool = typer.Option(False, "--on-paused", help="Subscribe to run pause events"),
    on_done: bool = typer.Option(True, "--on-done", help="Subscribe to run completion events"),
    on_error: bool = typer.Option(True, "--on-error", help="Subscribe to run error events"),
    active: bool = typer.Option(True, "--active/--inactive", help="Enable/disable webhook"),
):
    """Configure webhook delivery for a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.delivery.webhook(
        squid_id, url=url, is_active=active, retry=retry,
        on_running=on_running, on_paused=on_paused, on_done=on_done, on_error=on_error,
    )
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"Webhook delivery configured for {squid_id[:12]}")
    print_detail([
        ("URL", result.url),
        ("Active", result.is_active),
        ("Retry", result.retry),
    ])


@delivery_app.command("sftp")
def configure_sftp(
    squid: str = typer.Argument(..., help="Squid hash, name, or alias"),
    host: str = typer.Option(..., "--host", help="SFTP server hostname"),
    port: int = typer.Option(22, "--port", help="SFTP port"),
    username: str = typer.Option(..., "--username", help="SFTP username"),
    password: str = typer.Option(..., "--password", help="SFTP password"),
    directory: str = typer.Option(..., "--directory", help="Target directory on server"),
    active: bool = typer.Option(True, "--active/--inactive", help="Enable/disable SFTP"),
):
    """Configure SFTP delivery for a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.delivery.sftp(
        squid_id, host=host, port=port, username=username,
        password=password, directory=directory, is_active=active,
    )
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"SFTP delivery configured for {squid_id[:12]}")
    print_detail([
        ("Host", result.host),
        ("Port", result.port),
        ("Username", result.username),
        ("Directory", result.directory),
        ("Active", result.is_active),
    ])


# --- Test commands ---


@delivery_app.command("test-email")
def test_email(email: str = typer.Option(..., "--email", help="Email address to test")):
    """Test email delivery configuration."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    result = client.delivery.test_email(email=email)
    if _state.get("json"):
        print_json(result)
        return
    if result.get("success"):
        print_success("Email delivery test passed")
    else:
        print_error("Email delivery test failed")
        raise typer.Exit(1)


@delivery_app.command("test-googlesheet")
def test_googlesheet(url: str = typer.Option(..., "--url", help="Google Sheet URL to test")):
    """Test Google Sheet delivery configuration."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    result = client.delivery.test_google_sheet(url=url)
    if _state.get("json"):
        print_json(result)
        return
    if result.get("success"):
        print_success("Google Sheet delivery test passed")
    else:
        print_error("Google Sheet delivery test failed")
        raise typer.Exit(1)


@delivery_app.command("test-s3")
def test_s3(
    bucket: str = typer.Option(..., "--bucket", help="S3 bucket name"),
    aws_access_key: Optional[str] = typer.Option(None, "--aws-access-key"),
    aws_secret_key: Optional[str] = typer.Option(None, "--aws-secret-key"),
):
    """Test S3 delivery configuration."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    result = client.delivery.test_s3(bucket=bucket, aws_access_key=aws_access_key, aws_secret_key=aws_secret_key)
    if _state.get("json"):
        print_json(result)
        return
    if result.get("success"):
        print_success("S3 delivery test passed")
    else:
        print_error("S3 delivery test failed")
        raise typer.Exit(1)


@delivery_app.command("test-webhook")
def test_webhook(url: str = typer.Option(..., "--url", help="Webhook URL to test")):
    """Test webhook delivery configuration."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    result = client.delivery.test_webhook(url=url)
    if _state.get("json"):
        print_json(result)
        return
    if result.get("success"):
        print_success("Webhook delivery test passed")
    else:
        print_error("Webhook delivery test failed")
        raise typer.Exit(1)


@delivery_app.command("test-sftp")
def test_sftp(
    host: str = typer.Option(..., "--host"),
    port: int = typer.Option(22, "--port"),
    username: str = typer.Option(..., "--username"),
    password: str = typer.Option(..., "--password"),
    directory: str = typer.Option(..., "--directory"),
):
    """Test SFTP delivery configuration."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    result = client.delivery.test_sftp(
        host=host, port=port, username=username, password=password, directory=directory,
    )
    if _state.get("json"):
        print_json(result)
        return
    if result.get("success"):
        print_success("SFTP delivery test passed")
    else:
        print_error("SFTP delivery test failed")
        raise typer.Exit(1)



"""Tests for SDK client integration (error types and basic connectivity)."""
import pytest
from lobstrio.exceptions import APIError, AuthError, NotFoundError, RateLimitError


class TestErrorTypes:
    def test_auth_error_is_api_error(self):
        assert issubclass(AuthError, APIError)

    def test_not_found_error_is_api_error(self):
        assert issubclass(NotFoundError, APIError)

    def test_rate_limit_error_is_api_error(self):
        assert issubclass(RateLimitError, APIError)

    def test_api_error_has_status_code(self):
        e = APIError(400, "Bad request", {"error": "Bad request"})
        assert e.status_code == 400
        assert "Bad request" in str(e)

    def test_auth_error_message(self):
        e = AuthError(401, "Invalid token", {})
        assert e.status_code == 401
        assert "Invalid token" in str(e)


class TestClientImport:
    def test_can_import_client(self):
        from lobstrio import LobstrClient
        assert LobstrClient is not None

    def test_client_requires_token(self):
        from lobstrio import LobstrClient
        from unittest.mock import patch
        import os
        # Remove env var and mock config to ensure no token found
        old = os.environ.pop("LOBSTR_TOKEN", None)
        try:
            with patch("lobstrio.client._resolve_token", return_value=None):
                with pytest.raises(ValueError, match="No API token"):
                    LobstrClient(token=None)
        finally:
            if old:
                os.environ["LOBSTR_TOKEN"] = old



from __future__ import annotations

from dataclasses import asdict
from typing import Optional
import typer

from lobstr_cli.display import print_json, print_table, print_detail, print_success, print_error
from lobstr_cli.resolve import resolve_squid as _resolve_squid

squid_app = typer.Typer(no_args_is_help=True)


@squid_app.command("create")
def create_squid(
    crawler: str = typer.Argument(..., help="Crawler hash, prefix, or name"),
    name: Optional[str] = typer.Option(None, "--name", help="Custom squid name"),
):
    """Create a new squid for a crawler."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    all_crawlers = client.crawlers.list()
    from lobstr_cli.resolve import resolve_crawler
    crawler_id = resolve_crawler(crawler, all_crawlers)
    result = client.squids.create(crawler_id, name=name)
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"Created squid: {result.name} ({result.id[:12]})")


@squid_app.command("ls")
def list_squids(
    name: Optional[str] = typer.Option(None, "--name", help="Filter by name"),
    limit: int = typer.Option(50, "--limit"),
    page: int = typer.Option(1, "--page"),
):
    """List your squids."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    items = client.squids.list(limit=limit, page=page, name=name)
    if _state.get("json"):
        print_json([asdict(s) for s in items])
        return
    rows = []
    for s in items:
        rows.append([
            s.name,
            s.id[:12],
            s.crawler_name,
            "yes" if s.to_complete else "no",
            s.last_run_status or "—",
            str(s.concurrency),
        ])
    print_table(["Name", "Hash", "Crawler", "Pending", "Last Run", "Conc."], rows)


@squid_app.command("show")
def show_squid(squid: str = typer.Argument(..., help="Squid hash or prefix")):
    """Show squid details."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    data = client.squids.get(squid_id)
    if _state.get("json"):
        print_json(asdict(data))
        return
    print_detail([
        ("Name", data.name),
        ("Hash", data.id),
        ("Crawler", data.crawler_name),
        ("Active", data.is_active),
        ("Ready", data.is_ready),
        ("Concurrency", data.concurrency),
        ("Pending Tasks", data.to_complete),
        ("Last Run", data.last_run_status),
        ("Last Run At", data.last_run_at),
        ("Total Runs", data.total_runs),
        ("Unique Results", data.export_unique_results),
        ("Params", data.params),
    ])


@squid_app.command("update")
def update_squid(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    concurrency: Optional[int] = typer.Option(None, "--concurrency"),
    name: Optional[str] = typer.Option(None, "--name"),
    notify: Optional[str] = typer.Option(None, "--notify", help="on_success|on_error|null"),
    unique_results: Optional[bool] = typer.Option(None, "--unique-results/--no-unique-results"),
    param: Optional[list[str]] = typer.Option(None, "--param", help="KEY=VALUE, repeatable"),
):
    """Update squid configuration."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    kwargs: dict = {}
    if concurrency is not None:
        kwargs["concurrency"] = concurrency
    if name is not None:
        kwargs["name"] = name
    if notify is not None:
        kwargs["run_notify"] = None if notify == "null" else notify
    if unique_results is not None:
        kwargs["export_unique_results"] = unique_results
    if param:
        from lobstr_cli.resolve import parse_params
        kwargs["params"] = parse_params(param)
    if not kwargs:
        print_error("No options specified. Use --help to see available options.")
        raise typer.Exit(1)
    result = client.squids.update(squid_id, **kwargs)
    if _state.get("json"):
        print_json(asdict(result))
        return
    print_success(f"Updated squid {squid_id[:12]}")


@squid_app.command("empty")
def empty_squid(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    type: str = typer.Option("url", "--type", help="url or params"),
):
    """Remove all tasks from a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.squids.empty(squid_id, type=type)
    if _state.get("json"):
        print_json(result)
        return
    print_success(f"Emptied {result.get('deleted_count', '?')} tasks from {squid_id[:12]}")


@squid_app.command("rm")
def delete_squid(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    force: bool = typer.Option(False, "--force", "-f", help="Skip confirmation"),
):
    """Delete a squid permanently."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    if not force:
        typer.confirm(f"Delete squid {squid_id[:12]}? This is permanent.", abort=True)
    result = client.squids.delete(squid_id)
    if _state.get("json"):
        print_json(result)
        return
    print_success(f"Deleted squid {squid_id[:12]}")



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, get_client, _state


runner = CliRunner()


@pytest.fixture(autouse=True)
def clean_state():
    """Reset shared state before each test."""
    _state.clear()
    yield
    _state.clear()


class TestVersion:
    def test_version_flag(self):
        result = runner.invoke(app, ["--version"])
        assert result.exit_code == 0
        assert "lobstr" in result.output

    def test_no_args_shows_help(self):
        result = runner.invoke(app, [])
        assert result.exit_code in (0, 2)
        assert "Usage" in result.output or "lobstr" in result.output


class TestGlobalFlags:
    def test_json_flag_sets_state(self):
        with patch("lobstr_cli.cli.get_token", return_value="t"):
            result = runner.invoke(app, ["--json", "--version"])
        assert result.exit_code == 0

    def test_verbose_flag(self):
        result = runner.invoke(app, ["--verbose", "--version"])
        assert result.exit_code == 0

    def test_quiet_flag(self):
        result = runner.invoke(app, ["--quiet", "--version"])
        assert result.exit_code == 0


class TestGetClient:
    def test_no_token_exits(self):
        with patch("lobstr_cli.cli.get_token", return_value=None):
            with pytest.raises((SystemExit, Exception)):
                get_client()

    def test_creates_client_with_token(self):
        _state["token"] = "test-token-123"
        with patch("lobstr_cli.cli.get_token", return_value="test-token-123"):
            client = get_client()
        assert client is not None
        assert "client" in _state

    def test_reuses_existing_client(self):
        mock_client = MagicMock()
        _state["client"] = mock_client
        assert get_client() is mock_client


class TestSubcommands:
    def test_config_help(self):
        result = runner.invoke(app, ["config", "--help"])
        assert result.exit_code == 0
        assert "set-token" in result.output

    def test_crawlers_help(self):
        result = runner.invoke(app, ["crawlers", "--help"])
        assert result.exit_code == 0

    def test_squid_help(self):
        result = runner.invoke(app, ["squid", "--help"])
        assert result.exit_code == 0

    def test_task_help(self):
        result = runner.invoke(app, ["task", "--help"])
        assert result.exit_code == 0

    def test_run_help(self):
        result = runner.invoke(app, ["run", "--help"])
        assert result.exit_code == 0

    def test_results_help(self):
        result = runner.invoke(app, ["results", "--help"])
        assert result.exit_code == 0



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.crawler import Crawler, CrawlerAttribute, CrawlerParams


runner = CliRunner()

SAMPLE_CRAWLERS = [
    Crawler(
        id="abc123def456", name="Google Maps Leads Scraper",
        slug="google-maps-leads-scraper", description=None,
        credits_per_row=3, credits_per_email=None, max_concurrency=5,
        account=False, has_email_verification=False, is_public=True,
        is_premium=False, is_available=True, has_issues=False, rank=1,
    ),
    Crawler(
        id="def789abc012", name="LinkedIn Profile Scraper",
        slug="linkedin-profile-scraper", description=None,
        credits_per_row=2, credits_per_email=None, max_concurrency=3,
        account=True, has_email_verification=False, is_public=True,
        is_premium=True, is_available=True, has_issues=False, rank=2,
    ),
]


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client():
    mock = MagicMock()
    mock.crawlers.list.return_value = SAMPLE_CRAWLERS
    return mock


class TestCrawlersLs:
    def test_list_crawlers(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "ls"])
        assert result.exit_code == 0
        assert "Google Maps" in result.output
        assert "LinkedIn" in result.output

    def test_list_crawlers_json(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "crawlers", "ls"])
        assert result.exit_code == 0
        assert "abc123def456" in result.output

    def test_shows_status_flags(self):
        crawlers = [
            Crawler(
                id="a1", name="C1", slug="c1", description=None,
                credits_per_row=1, credits_per_email=None, max_concurrency=1,
                account=False, has_email_verification=False, is_public=True,
                is_premium=False, is_available=True, has_issues=True, rank=None,
            ),
            Crawler(
                id="a2", name="C2", slug="c2", description=None,
                credits_per_row=1, credits_per_email=None, max_concurrency=1,
                account=False, has_email_verification=False, is_public=True,
                is_premium=False, is_available=False, has_issues=False, rank=None,
            ),
        ]
        mock = MagicMock()
        mock.crawlers.list.return_value = crawlers
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "ls"])
        assert result.exit_code == 0


class TestCrawlersShow:
    CRAWLER_DETAIL = Crawler(
        id="abc123def456", name="Google Maps Leads Scraper",
        slug="google-maps-leads-scraper", description="Scrape Google Maps",
        credits_per_row=3, credits_per_email=None, max_concurrency=5,
        account=False, has_email_verification=False, is_public=True,
        is_premium=False, is_available=True, has_issues=False, rank=1,
        input_params=[{"name": "url", "level": "task", "type": "string", "required": "true", "default": ""}],
        result_fields=["name", "address", "phone"],
    )

    def test_show_by_slug(self):
        mock = _mock_client()
        mock.crawlers.get.return_value = self.CRAWLER_DETAIL
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "show", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "Google Maps Leads Scraper" in result.output
        assert "google-maps-leads-scraper" in result.output

    def test_show_by_name(self):
        detail = Crawler(
            id="def789abc012", name="LinkedIn Profile Scraper",
            slug="linkedin-profile-scraper", description=None,
            credits_per_row=2, credits_per_email=None, max_concurrency=3,
            account=True, has_email_verification=False, is_public=True,
            is_premium=True, is_available=True, has_issues=False, rank=2,
        )
        mock = _mock_client()
        mock.crawlers.get.return_value = detail
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "show", "LinkedIn Profile"])
        assert result.exit_code == 0
        assert "LinkedIn Profile Scraper" in result.output

    def test_show_json(self):
        mock = _mock_client()
        mock.crawlers.get.return_value = self.CRAWLER_DETAIL
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "crawlers", "show", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "abc123def456" in result.output

    def test_show_displays_input_params(self):
        mock = _mock_client()
        mock.crawlers.get.return_value = self.CRAWLER_DETAIL
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "show", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "url" in result.output

    def test_show_displays_result_fields(self):
        mock = _mock_client()
        mock.crawlers.get.return_value = self.CRAWLER_DETAIL
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "show", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "phone" in result.output
        assert "name, address, phone" in result.output


class TestCrawlersSearch:
    def test_search_finds_match(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "search", "Google"])
        assert result.exit_code == 0
        assert "Google Maps" in result.output

    def test_search_no_match(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "search", "Facebook"])
        assert result.exit_code == 0

    def test_search_json(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "crawlers", "search", "LinkedIn"])
        assert result.exit_code == 0
        assert "LinkedIn" in result.output


class TestCrawlersParams:
    def test_shows_params(self):
        mock = _mock_client()
        mock.crawlers.params.return_value = CrawlerParams(
            task_params={"url": {"type": "string", "required": True, "default": "", "regex": ""}},
            squid_params={"max_results": {"default": 100, "required": False, "allowed": [50, 100, 200]}},
            functions={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "params", "Google Maps"])
        assert result.exit_code == 0
        assert "url" in result.output

    def test_params_with_functions(self):
        mock = _mock_client()
        mock.crawlers.params.return_value = CrawlerParams(
            task_params={},
            squid_params={},
            functions={"email_finder": {"credits_per_function": {"current": 5}, "default": False}},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "params", "Google Maps"])
        assert result.exit_code == 0

    def test_params_json(self):
        mock = _mock_client()
        mock.crawlers.params.return_value = CrawlerParams(
            task_params={}, squid_params={}, functions={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "crawlers", "params", "Google Maps"])
        assert result.exit_code == 0

    def test_allowed_with_int_values(self):
        mock = _mock_client()
        mock.crawlers.params.return_value = CrawlerParams(
            task_params={},
            squid_params={"max_results": {"default": 100, "required": False, "allowed": [10, 20, 50, 100]}},
            functions={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "params", "Google Maps"])
        assert result.exit_code == 0
        assert "10" in result.output

    def test_params_by_exact_slug(self):
        mock = _mock_client()
        mock.crawlers.params.return_value = CrawlerParams(
            task_params={"url": {"type": "string", "required": True, "default": "", "regex": ""}},
            squid_params={}, functions={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "params", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "url" in result.output

    def test_params_by_slug_prefix(self):
        mock = _mock_client()
        mock.crawlers.params.return_value = CrawlerParams(
            task_params={"url": {"type": "string", "required": True, "default": "", "regex": ""}},
            squid_params={}, functions={},
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "params", "linkedin-profile"])
        assert result.exit_code == 0


class TestCrawlersAttrs:
    SAMPLE_ATTRS = [
        CrawlerAttribute(
            name="name", type="string", example="Blues Billard Club",
            function="Export Local Businesses", is_main=True,
            description="Business name as listed on Google Maps",
        ),
        CrawlerAttribute(
            name="email", type="string", example="info@lobstr.io",
            function="Extract Emails from Website", is_main=False,
            description="Email address found on the business website",
        ),
    ]

    def test_attrs_table(self):
        mock = _mock_client()
        mock.crawlers.attributes.return_value = self.SAMPLE_ATTRS
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "attrs", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "name" in result.output
        assert "email" in result.output
        # Grouped by function
        assert "Export Local Businesses" in result.output
        assert "Extract Emails from Website" in result.output

    def test_attrs_json(self):
        mock = _mock_client()
        mock.crawlers.attributes.return_value = self.SAMPLE_ATTRS
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "crawlers", "attrs", "google-maps-leads-scraper"])
        assert result.exit_code == 0
        assert "Blues Billard Club" in result.output

    def test_attrs_empty(self):
        mock = _mock_client()
        mock.crawlers.attributes.return_value = []
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "attrs", "google-maps-leads-scraper"])
        assert result.exit_code == 0


class TestCrawlersParamsAmbiguous:
    def test_params_ambiguous_slug(self):
        crawlers = [
            Crawler(
                id="c1", name="Google Maps Leads", slug="google-maps-leads",
                description=None, credits_per_row=1, credits_per_email=None,
                max_concurrency=1, account=False, has_email_verification=False,
                is_public=True, is_premium=False, is_available=True, has_issues=False, rank=None,
            ),
            Crawler(
                id="c2", name="Google Maps Reviews", slug="google-maps-reviews",
                description=None, credits_per_row=1, credits_per_email=None,
                max_concurrency=1, account=False, has_email_verification=False,
                is_public=True, is_premium=False, is_available=True, has_issues=False, rank=None,
            ),
        ]
        mock = MagicMock()
        mock.crawlers.list.return_value = crawlers
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["crawlers", "params", "google-maps"])
        assert result.exit_code != 0



import pytest
from unittest.mock import MagicMock
from lobstr_cli.resolve import (
    match_hash_prefix,
    match_slug,
    match_name,
    match_username,
    match_crawler_name,
    resolve_crawler,
    resolve_squid,
    resolve_account,
    parse_param_value,
    parse_params,
    require_full_hash,
)


# --- match_hash_prefix ---

class TestMatchHashPrefix:
    def test_exact_match(self):
        items = [{"id": "abc123"}, {"id": "abc456"}, {"id": "def789"}]
        assert match_hash_prefix("abc123", items) == "abc123"

    def test_unique_prefix(self):
        items = [{"id": "abc123"}, {"id": "def456"}]
        assert match_hash_prefix("abc", items) == "abc123"

    def test_ambiguous_prefix(self):
        items = [{"id": "abc123"}, {"id": "abc456"}]
        with pytest.raises(SystemExit):
            match_hash_prefix("abc", items)

    def test_no_match(self):
        items = [{"id": "abc123"}]
        with pytest.raises(SystemExit):
            match_hash_prefix("xyz", items)

    def test_custom_key(self):
        items = [{"hash": "abc123"}, {"hash": "def456"}]
        assert match_hash_prefix("abc", items, key="hash") == "abc123"

    def test_empty_items(self):
        with pytest.raises(SystemExit):
            match_hash_prefix("abc", [])

    def test_single_char_prefix(self):
        items = [{"id": "abc123"}, {"id": "def456"}]
        assert match_hash_prefix("a", items) == "abc123"

    def test_full_hash_exact_among_prefixes(self):
        """Exact match should win even if it's a prefix of another."""
        items = [{"id": "abc"}, {"id": "abc123"}]
        assert match_hash_prefix("abc", items) == "abc"

    def test_many_ambiguous_shows_max_five(self, capsys):
        items = [{"id": f"abc{i:03d}"} for i in range(10)]
        with pytest.raises(SystemExit):
            match_hash_prefix("abc", items)


# --- match_slug ---

class TestMatchSlug:
    def test_exact_slug(self):
        items = [
            {"id": "s1", "slug": "google-maps-leads-scraper", "name": "Google Maps Leads Scraper"},
            {"id": "s2", "slug": "linkedin-profile-scraper", "name": "LinkedIn Profile Scraper"},
        ]
        assert match_slug("google-maps-leads-scraper", items) == "s1"

    def test_slug_prefix(self):
        items = [
            {"id": "s1", "slug": "google-maps-leads-scraper", "name": "Google Maps Leads Scraper"},
            {"id": "s2", "slug": "linkedin-profile-scraper", "name": "LinkedIn Profile Scraper"},
        ]
        assert match_slug("google-maps", items) == "s1"

    def test_ambiguous_slug(self):
        items = [
            {"id": "s1", "slug": "google-maps-leads", "name": "Google Maps Leads"},
            {"id": "s2", "slug": "google-maps-reviews", "name": "Google Maps Reviews"},
        ]
        with pytest.raises(SystemExit):
            match_slug("google-maps", items)

    def test_no_match(self):
        items = [{"id": "s1", "slug": "google-maps-leads", "name": "Google Maps Leads"}]
        with pytest.raises(SystemExit):
            match_slug("facebook-ads", items)

    def test_empty_list(self):
        with pytest.raises(SystemExit):
            match_slug("anything", [])

    def test_exact_wins_over_prefix(self):
        items = [
            {"id": "s1", "slug": "google-maps", "name": "Google Maps"},
            {"id": "s2", "slug": "google-maps-leads-scraper", "name": "Google Maps Leads Scraper"},
        ]
        assert match_slug("google-maps", items) == "s1"

    def test_case_insensitive(self):
        items = [{"id": "s1", "slug": "google-maps-leads", "name": "Google Maps Leads"}]
        assert match_slug("Google-Maps-Leads", items) == "s1"


# --- match_name ---

class TestMatchName:
    def test_exact_match(self):
        items = [{"id": "s1", "name": "My Leads"}, {"id": "s2", "name": "My Reviews"}]
        assert match_name("My Leads", items) == "s1"

    def test_case_insensitive(self):
        items = [{"id": "s1", "name": "My Leads Scraper"}]
        assert match_name("my leads scraper", items) == "s1"

    def test_substring_match(self):
        items = [{"id": "s1", "name": "Google Maps Leads"}, {"id": "s2", "name": "LinkedIn Profiles"}]
        assert match_name("maps leads", items) == "s1"

    def test_ambiguous_name(self):
        items = [{"id": "s1", "name": "Maps Leads"}, {"id": "s2", "name": "Maps Reviews"}]
        with pytest.raises(SystemExit):
            match_name("Maps", items)

    def test_no_match(self):
        items = [{"id": "s1", "name": "My Leads"}]
        with pytest.raises(SystemExit):
            match_name("facebook", items)

    def test_empty_list(self):
        with pytest.raises(SystemExit):
            match_name("anything", [])

    def test_exact_wins_over_substring(self):
        items = [{"id": "s1", "name": "Maps"}, {"id": "s2", "name": "Google Maps Leads"}]
        assert match_name("Maps", items) == "s1"


# --- resolve_squid ---

class TestResolveSquid:
    def _mock_client(self, squids):
        mock = MagicMock()
        mock.squids.list.return_value = squids
        return mock

    def test_resolve_by_hash_prefix(self):
        client = self._mock_client([{"id": "abc123def456", "name": "My Scraper"}])
        assert resolve_squid(client, "abc123") == "abc123def456"

    def test_resolve_by_exact_name(self):
        client = self._mock_client([
            {"id": "s1", "name": "Maps"},
            {"id": "s2", "name": "Google Maps Leads"},
        ])
        assert resolve_squid(client, "Maps") == "s1"

    def test_resolve_by_name_substring(self):
        client = self._mock_client([{"id": "s1", "name": "My Leads Scraper"}])
        assert resolve_squid(client, "My Leads") == "s1"

    def test_resolve_name_case_insensitive(self):
        client = self._mock_client([{"id": "s1", "name": "My Leads"}])
        assert resolve_squid(client, "my leads") == "s1"

    def test_resolve_alias(self):
        client = self._mock_client([{"id": "abc123def456", "name": "Scraper"}])
        from unittest.mock import patch
        with patch("lobstr_cli.config.resolve_alias", return_value="abc123def456"):
            assert resolve_squid(client, "@maps") == "abc123def456"

    def test_hash_first_priority(self):
        """Pure hex input should try hash before name."""
        client = self._mock_client([{"id": "deadbeef1234", "name": "Other"}])
        assert resolve_squid(client, "deadbeef") == "deadbeef1234"

    def test_no_match_raises(self):
        client = self._mock_client([{"id": "abc123", "name": "My Scraper"}])
        with pytest.raises(SystemExit):
            resolve_squid(client, "nonexistent")

    def test_ambiguous_name_raises(self):
        client = self._mock_client([
            {"id": "s1", "name": "Google Maps Leads"},
            {"id": "s2", "name": "Google Maps Reviews"},
        ])
        with pytest.raises(SystemExit):
            resolve_squid(client, "Google Maps")


# --- match_crawler_name ---

class TestMatchCrawlerName:
    def test_exact_match(self):
        crawlers = [
            {"id": "c1", "name": "LinkedIn Profile Scraper"},
            {"id": "c2", "name": "Google Maps Reviews"},
        ]
        assert match_crawler_name("LinkedIn Profile Scraper", crawlers) == "c1"

    def test_case_insensitive_exact(self):
        crawlers = [{"id": "c1", "name": "Google Maps Leads Scraper"}]
        assert match_crawler_name("google maps leads scraper", crawlers) == "c1"

    def test_substring_match(self):
        crawlers = [
            {"id": "c1", "name": "LinkedIn Profile Scraper"},
            {"id": "c2", "name": "Google Maps Reviews"},
        ]
        assert match_crawler_name("linkedin profile", crawlers) == "c1"

    def test_ambiguous_name(self):
        crawlers = [
            {"id": "c1", "name": "LinkedIn Profile Scraper"},
            {"id": "c2", "name": "LinkedIn Company Scraper"},
        ]
        with pytest.raises(SystemExit):
            match_crawler_name("linkedin", crawlers)

    def test_no_match(self):
        crawlers = [{"id": "c1", "name": "LinkedIn Profile Scraper"}]
        with pytest.raises(SystemExit):
            match_crawler_name("facebook", crawlers)

    def test_empty_list(self):
        with pytest.raises(SystemExit):
            match_crawler_name("anything", [])

    def test_partial_word_match(self):
        crawlers = [
            {"id": "c1", "name": "Google Maps Leads Scraper"},
            {"id": "c2", "name": "Google Maps Reviews Scraper"},
        ]
        assert match_crawler_name("Maps Leads", crawlers) == "c1"


# --- resolve_crawler ---

class TestResolveCrawler:
    def test_resolve_by_hash(self):
        crawlers = [
            {"id": "abc123def456", "name": "Some Crawler", "slug": "some-crawler"},
            {"id": "def789abc012", "name": "Other Crawler", "slug": "other-crawler"},
        ]
        assert resolve_crawler("abc123", crawlers) == "abc123def456"

    def test_resolve_by_slug(self):
        crawlers = [{"id": "c1", "name": "Google Maps Leads Scraper", "slug": "google-maps-leads-scraper"}]
        assert resolve_crawler("google-maps-leads-scraper", crawlers) == "c1"

    def test_resolve_by_slug_prefix(self):
        crawlers = [
            {"id": "c1", "name": "Google Maps Leads Scraper", "slug": "google-maps-leads-scraper"},
            {"id": "c2", "name": "LinkedIn Profile Scraper", "slug": "linkedin-profile-scraper"},
        ]
        assert resolve_crawler("google-maps", crawlers) == "c1"

    def test_resolve_by_name(self):
        crawlers = [{"id": "abc123def456", "name": "Google Maps Leads Scraper", "slug": "google-maps-leads-scraper"}]
        assert resolve_crawler("Google Maps", crawlers) == "abc123def456"

    def test_hash_fallback_to_name(self):
        crawlers = [{"id": "abc123", "name": "My Scraper", "slug": "my-scraper"}]
        assert resolve_crawler("My Scraper", crawlers) == "abc123"

    def test_hex_like_tries_hash_first(self):
        crawlers = [{"id": "xyz789", "name": "deadbeef", "slug": "deadbeef"}]
        assert resolve_crawler("deadbeef", crawlers) == "xyz789"

    def test_resolve_full_hash(self):
        crawlers = [{"id": "abc123def456789012345678", "name": "Crawler", "slug": "crawler"}]
        assert resolve_crawler("abc123def456789012345678", crawlers) == "abc123def456789012345678"


# --- parse_param_value ---

class TestParseParamValue:
    def test_integer(self):
        assert parse_param_value("42") == 42

    def test_negative_integer(self):
        assert parse_param_value("-5") == -5

    def test_float(self):
        assert parse_param_value("3.14") == 3.14

    def test_true(self):
        assert parse_param_value("true") is True

    def test_false(self):
        assert parse_param_value("false") is False

    def test_true_uppercase(self):
        assert parse_param_value("True") is True

    def test_false_mixed_case(self):
        assert parse_param_value("FALSE") is False

    def test_none(self):
        assert parse_param_value("none") is None

    def test_none_uppercase(self):
        assert parse_param_value("None") is None

    def test_string(self):
        assert parse_param_value("hello world") == "hello world"

    def test_empty_string(self):
        assert parse_param_value("") == ""

    def test_string_with_numbers(self):
        assert parse_param_value("abc123") == "abc123"

    def test_zero(self):
        assert parse_param_value("0") == 0

    def test_float_zero(self):
        assert parse_param_value("0.0") == 0.0


# --- parse_params ---

class TestParseParams:
    def test_single_param(self):
        assert parse_params(["max_results=50"]) == {"max_results": 50}

    def test_multiple_params(self):
        result = parse_params(["max_results=50", "language=English", "geo_match=true"])
        assert result == {"max_results": 50, "language": "English", "geo_match": True}

    def test_param_with_equals_in_value(self):
        result = parse_params(["url=https://example.com?a=1"])
        assert result == {"url": "https://example.com?a=1"}

    def test_empty_value(self):
        result = parse_params(["key="])
        assert result == {"key": ""}

    def test_none_value(self):
        result = parse_params(["key=none"])
        assert result == {"key": None}

    def test_empty_list(self):
        assert parse_params([]) == {}

    def test_overwrite_duplicate_key(self):
        result = parse_params(["key=first", "key=second"])
        assert result == {"key": "second"}


# --- require_full_hash ---

class TestRequireFullHash:
    def test_valid_32_char_hash(self):
        # Should not raise
        require_full_hash("a" * 32, "run")

    def test_short_hash_raises(self):
        with pytest.raises(SystemExit):
            require_full_hash("abc123def456", "run")

    def test_12_char_hash_raises(self):
        with pytest.raises(SystemExit):
            require_full_hash("26a4377e8b37", "task")

    def test_empty_hash_raises(self):
        with pytest.raises(SystemExit):
            require_full_hash("", "run")

    def test_31_char_hash_raises(self):
        with pytest.raises(SystemExit):
            require_full_hash("a" * 31, "run")

    def test_longer_than_32_passes(self):
        # Should not raise for >= 32
        require_full_hash("a" * 40, "run")


# --- match_username ---

class TestMatchUsername:
    def test_exact_match(self):
        items = [{"id": "a1", "username": "johndoe"}, {"id": "a2", "username": "janedoe"}]
        assert match_username("johndoe", items) == "a1"

    def test_case_insensitive(self):
        items = [{"id": "a1", "username": "JohnDoe"}]
        assert match_username("johndoe", items) == "a1"

    def test_substring_match(self):
        items = [{"id": "a1", "username": "john_twitter"}, {"id": "a2", "username": "jane_facebook"}]
        assert match_username("twitter", items) == "a1"

    def test_no_match(self):
        items = [{"id": "a1", "username": "johndoe"}]
        with pytest.raises(SystemExit):
            match_username("notfound", items)

    def test_ambiguous_match(self):
        items = [{"id": "a1", "username": "john_twitter"}, {"id": "a2", "username": "john_facebook"}]
        with pytest.raises(SystemExit):
            match_username("john", items)

    def test_exact_wins_over_substring(self):
        items = [
            {"id": "a1", "username": "john"},
            {"id": "a2", "username": "john_doe"},
        ]
        assert match_username("john", items) == "a1"


# --- resolve_account ---

class TestResolveAccount:
    def test_resolve_by_hash_prefix(self):
        client = MagicMock()
        client.accounts.list.return_value = [{"id": "aabb11cc22dd", "username": "johndoe"}]
        assert resolve_account(client, "aabb11") == "aabb11cc22dd"

    def test_resolve_by_username(self):
        client = MagicMock()
        client.accounts.list.return_value = [{"id": "acc123", "username": "johndoe"}]
        assert resolve_account(client, "johndoe") == "acc123"

    def test_hex_falls_back_to_username(self):
        client = MagicMock()
        client.accounts.list.return_value = [{"id": "xyz789", "username": "deadbeef"}]
        assert resolve_account(client, "deadbeef") == "xyz789"

    def test_hex_prefers_hash_over_username(self):
        client = MagicMock()
        client.accounts.list.return_value = [{"id": "deadbeef1234", "username": "deadbeef"}]
        assert resolve_account(client, "deadbeef") == "deadbeef1234"



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.account import Account, AccountType, SyncStatus


runner = CliRunner()

ACCOUNTS = [
    Account(
        id="acc1abc123def456", username="johndoe", type="twitter-sync",
        status_code_info="synchronized", status_code_description="Account is synced",
        baseurl="https://twitter.com", created_at="2025-01-01T00:00:00Z",
        updated_at=None, last_synchronization_time="2025-01-15T10:00:00Z",
        squids=[{"name": "Twitter Scraper", "hash_value": "squid1"}],
        params={},
    ),
]

ACCOUNT_TYPES = [
    AccountType(
        name="twitter-sync", domain="Twitter",
        baseurl="https://twitter.com",
        cookies=[{"name": "auth_token", "required": True}],
    ),
]


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client():
    mock = MagicMock()
    mock.accounts.list.return_value = ACCOUNTS
    return mock


class TestAccountsLs:
    def test_list_accounts(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "ls"])
        assert result.exit_code == 0
        assert "johndoe" in result.output

    def test_list_json(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "accounts", "ls"])
        assert result.exit_code == 0
        assert "acc1abc123def456" in result.output


class TestAccountsShow:
    def test_show_by_username(self):
        mock = _mock_client()
        mock.accounts.get.return_value = ACCOUNTS[0]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "show", "johndoe"])
        assert result.exit_code == 0
        assert "johndoe" in result.output
        assert "twitter-sync" in result.output

    def test_show_json(self):
        mock = _mock_client()
        mock.accounts.get.return_value = ACCOUNTS[0]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "accounts", "show", "johndoe"])
        assert result.exit_code == 0
        assert "acc1abc123def456" in result.output


class TestAccountsRm:
    def test_delete_with_force(self):
        mock = _mock_client()
        mock.accounts.delete.return_value = {"id": "acc1abc123def456", "deleted": "true"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "rm", "johndoe", "--force"])
        assert result.exit_code == 0
        assert "Deleted" in result.output

    def test_delete_aborted(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "rm", "johndoe"], input="n\n")
        assert result.exit_code != 0


class TestAccountsTypes:
    def test_list_types(self):
        mock = _mock_client()
        mock.accounts.types.return_value = ACCOUNT_TYPES
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "types"])
        assert result.exit_code == 0
        assert "twitter-sync" in result.output
        assert "Twitter" in result.output

    def test_types_json(self):
        mock = _mock_client()
        mock.accounts.types.return_value = ACCOUNT_TYPES
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "accounts", "types"])
        assert result.exit_code == 0


class TestAccountsSync:
    def test_sync_new(self):
        mock = _mock_client()
        mock.accounts.sync.return_value = {"id": "sync123", "status_text": "created"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "sync", "twitter-sync", "--cookie", "auth_token=abc123"])
        assert result.exit_code == 0
        assert "Sync started" in result.output
        mock.accounts.sync.assert_called_once_with("twitter-sync", {"auth_token": "abc123"}, account=None)

    def test_sync_refresh(self):
        mock = _mock_client()
        mock.accounts.sync.return_value = {"id": "sync123", "status_text": "created"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, [
                "accounts", "sync", "twitter-sync",
                "--account", "acc1abc123def456",
                "--cookie", "auth_token=newtoken",
            ])
        assert result.exit_code == 0
        mock.accounts.sync.assert_called_once_with(
            "twitter-sync", {"auth_token": "newtoken"}, account="acc1abc123def456",
        )

    def test_sync_json_cookies(self):
        mock = _mock_client()
        mock.accounts.sync.return_value = {"id": "sync123", "status_text": "created"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, [
                "accounts", "sync", "twitter-sync",
                "--cookies-json", '{"auth_token": "abc", "ct0": "xyz"}',
            ])
        assert result.exit_code == 0
        call_args = mock.accounts.sync.call_args
        assert call_args[0][1]["auth_token"] == "abc"
        assert call_args[0][1]["ct0"] == "xyz"

    def test_sync_file_cookies(self, tmp_path):
        cookie_file = tmp_path / "cookies.json"
        cookie_file.write_text('{"auth_token": "fromfile"}')
        mock = _mock_client()
        mock.accounts.sync.return_value = {"id": "sync123", "status_text": "created"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, [
                "accounts", "sync", "twitter-sync",
                "--cookies-file", str(cookie_file),
            ])
        assert result.exit_code == 0
        call_args = mock.accounts.sync.call_args
        assert call_args[0][1]["auth_token"] == "fromfile"

    def test_sync_no_cookies_error(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "sync", "twitter-sync"])
        assert result.exit_code != 0

    def test_sync_json_mode(self):
        mock = _mock_client()
        mock.accounts.sync.return_value = {"id": "sync123", "status_text": "created"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "accounts", "sync", "twitter-sync", "--cookie", "auth_token=abc"])
        assert result.exit_code == 0
        assert "sync123" in result.output


class TestAccountsSyncStatus:
    def test_sync_status(self):
        mock = _mock_client()
        mock.accounts.sync_status.return_value = SyncStatus(
            id="sync123", status_code="200",
            status_text="synchronized", account_hash="acc1abc123def456",
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "sync-status", "sync123"])
        assert result.exit_code == 0
        assert "synchronized" in result.output


class TestAccountsUpdate:
    def test_update_limits(self):
        mock = _mock_client()
        mock.accounts.update.return_value = {"params": {"default": {}, "user": {}}}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, [
                "accounts", "update", "johndoe",
                "--type", "sales-nav-sync",
                "--param", "batch=20",
                "--param", "profiles=150",
            ])
        assert result.exit_code == 0
        mock.accounts.update.assert_called_once_with(
            "acc1abc123def456", type="sales-nav-sync",
            params={"batch": 20, "profiles": 150},
        )

    def test_update_no_params_error(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["accounts", "update", "johndoe", "--type", "twitter-sync"])
        assert result.exit_code != 0



from __future__ import annotations

import time
from dataclasses import asdict
from typing import Optional
import typer

from lobstr_cli.display import (
    print_json, print_table, print_detail, print_success,
    print_info, print_error, make_progress,
)
from lobstr_cli.resolve import resolve_squid as _resolve_squid, require_full_hash

run_app = typer.Typer(no_args_is_help=True)


def _poll_run(client, run_id: str, download_path: str | None = None):
    """Poll a run until completion, showing a progress bar."""
    with make_progress() as progress:
        task = progress.add_task("Running...", total=100)
        while True:
            stats = client.runs.stats(run_id)
            pct_str = stats.percent_done.replace("%", "")
            try:
                pct = float(pct_str)
            except ValueError:
                pct = 0
            progress.update(task, completed=pct, total=100,
                          description=f"Running... {stats.total_tasks_done}/{stats.total_tasks} tasks  ETA: {stats.eta or '?'}")
            if stats.is_done:
                break
            time.sleep(3)

    run = client.runs.get(run_id)
    if download_path:
        try:
            client.runs.download(run_id, download_path)
            print_success(f"Downloaded results to {download_path}")
        except Exception:
            print_error("No download URL available. Run may not have finished exporting.")
    return run


@run_app.command("start")
def start_run(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    wait: bool = typer.Option(False, "--wait", help="Poll until run completes"),
    download: Optional[str] = typer.Option(None, "--download", help="Download CSV on completion (implies --wait)"),
):
    """Start a new run."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    result = client.runs.start(squid=squid_id)
    if _state.get("json") and not (wait or download):
        print_json(asdict(result))
        return
    print_success(f"Started run {result.id}")
    if download or wait:
        try:
            run = _poll_run(client, result.id, download_path=download)
        except KeyboardInterrupt:
            print_info(f"\nInterrupted. Resume with: lobstr run watch {result.id}")
            raise typer.Exit(0)
        if _state.get("json"):
            print_json(asdict(run))
            return
        print_detail([
            ("Status", run.status),
            ("Results", run.total_results),
            ("Duration", run.duration),
            ("Credits", run.credit_used),
            ("Done Reason", run.done_reason),
        ])


@run_app.command("ls")
def list_runs(
    squid: str = typer.Argument(..., help="Squid hash or prefix"),
    limit: int = typer.Option(20, "--limit"),
    page: int = typer.Option(1, "--page"),
):
    """List runs for a squid."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    squid_id = _resolve_squid(client, squid)
    items = client.runs.list(squid=squid_id, limit=limit, page=page)
    if _state.get("json"):
        print_json([asdict(r) for r in items])
        return
    rows = []
    for r in items:
        dur_str = f"{r.duration:.0f}s" if isinstance(r.duration, (int, float)) else str(r.duration or "—")
        rows.append([
            r.id,
            r.status,
            str(r.total_results),
            dur_str,
            str(r.credit_used),
            r.done_reason or "—",
        ])
    print_table(["Hash", "Status", "Results", "Duration", "Credits", "Reason"], rows)


@run_app.command("show")
def show_run(run_id: str = typer.Argument(..., help="Full run hash")):
    """Show run details."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(run_id, "run")
    data = client.runs.get(run_id)
    if _state.get("json"):
        print_json(asdict(data))
        return
    print_detail([
        ("Hash", data.id),
        ("Status", data.status),
        ("Results", data.total_results),
        ("Unique Results", data.total_unique_results),
        ("Duration", data.duration),
        ("Credits", data.credit_used),
        ("Origin", data.origin),
        ("Done Reason", data.done_reason),
        ("Done Reason Desc", data.done_reason_desc),
        ("Export Done", data.export_done),
        ("Started At", data.started_at),
        ("Ended At", data.ended_at),
    ])


@run_app.command("stats")
def run_stats(run_id: str = typer.Argument(..., help="Full run hash")):
    """Show run statistics."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(run_id, "run")
    data = client.runs.stats(run_id)
    if _state.get("json"):
        print_json(asdict(data))
        return
    print_detail([
        ("Percent Done", data.percent_done),
        ("Tasks", f"{data.total_tasks_done}/{data.total_tasks}"),
        ("Tasks Left", data.total_tasks_left),
        ("Results", data.total_results),
        ("Duration", data.duration),
        ("ETA", data.eta),
        ("Current Task", data.current_task),
    ])


@run_app.command("tasks")
def run_tasks(
    run_id: str = typer.Argument(..., help="Full run hash"),
    limit: int = typer.Option(50, "--limit"),
    page: int = typer.Option(1, "--page"),
):
    """List tasks in a run."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(run_id, "run")
    items = client.runs.tasks(run_id, limit=limit, page=page)
    if _state.get("json"):
        print_json([asdict(t) for t in items])
        return
    rows = []
    for t in items:
        url = t.params.get("url", str(t.params)[:50])
        status = t.status.status if t.status else "—"
        results = str(t.status.total_results) if t.status else ""
        done_reason = (t.status.done_reason if t.status else None) or "—"
        rows.append([
            t.id,
            status,
            results,
            str(url)[:60],
            done_reason,
        ])
    print_table(["Hash", "Status", "Results", "URL/Params", "Reason"], rows)


@run_app.command("abort")
def abort_run(run_id: str = typer.Argument(..., help="Full run hash")):
    """Abort a running run."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(run_id, "run")
    result = client.runs.abort(run_id)
    if _state.get("json"):
        print_json(result)
        return
    print_success(f"Aborted run {run_id}")


@run_app.command("download")
def download_run(
    run_id: str = typer.Argument(..., help="Full run hash"),
    path: str = typer.Argument("results.csv", help="Output file path"),
):
    """Download run results as CSV."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(run_id, "run")
    if _state.get("json"):
        url = client.runs.download_url(run_id)
        print_json({"s3": url})
        return
    try:
        client.runs.download(run_id, path)
    except (KeyError, Exception):
        print_error("No download URL available. Run may not have finished exporting.")
        raise typer.Exit(1)
    print_success(f"Downloaded to {path}")


@run_app.command("watch")
def watch_run(run_id: str = typer.Argument(..., help="Full run hash")):
    """Live-poll run progress with progress bar."""
    from lobstr_cli.cli import get_client, _state
    client = get_client()
    require_full_hash(run_id, "run")
    try:
        run = _poll_run(client, run_id)
    except KeyboardInterrupt:
        print_info("\nStopped watching.")
        raise typer.Exit(0)
    if _state.get("json"):
        print_json(asdict(run))
        return
    print_detail([
        ("Status", run.status),
        ("Results", run.total_results),
        ("Duration", run.duration),
        ("Credits", run.credit_used),
        ("Done Reason", run.done_reason),
    ])



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.squid import Squid
from lobstrio.models.task import Task, TaskStatus, AddTasksResult, UploadStatus, UploadMeta


runner = CliRunner()

SQUIDS = [
    Squid(
        id="squid1abc123def456", name="My Squid", crawler="c1",
        crawler_name="Google Maps", is_active=True, is_ready=True,
        concurrency=1, to_complete=None, last_run_status=None,
        last_run_at=None, total_runs=0, export_unique_results=False,
        params={},
    ),
]

FULL_TASK_HASH = "a" * 32


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client():
    mock = MagicMock()
    mock.squids.list.return_value = SQUIDS
    return mock


class TestTaskAdd:
    def test_add_tasks(self):
        mock = _mock_client()
        mock.tasks.add.return_value = AddTasksResult(
            tasks=[Task(id="t1", is_active=True, params={"url": "https://a.com"}, status=None, created_at=None),
                   Task(id="t2", is_active=True, params={"url": "https://b.com"}, status=None, created_at=None)],
            duplicated_count=0,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "add", "My Squid", "https://a.com", "https://b.com"])
        assert result.exit_code == 0
        assert "Added 2 tasks" in result.output

    def test_add_with_custom_key(self):
        mock = _mock_client()
        mock.tasks.add.return_value = AddTasksResult(
            tasks=[Task(id="t1", is_active=True, params={"keyword": "pizza nyc"}, status=None, created_at=None)],
            duplicated_count=0,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "add", "My Squid", "pizza nyc", "--key", "keyword"])
        assert result.exit_code == 0
        mock.tasks.add.assert_called_once_with(squid="squid1abc123def456", tasks=[{"keyword": "pizza nyc"}])

    def test_add_with_duplicates(self):
        mock = _mock_client()
        mock.tasks.add.return_value = AddTasksResult(
            tasks=[Task(id="t1", is_active=True, params={}, status=None, created_at=None)],
            duplicated_count=3,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "add", "My Squid", "https://a.com"])
        assert result.exit_code == 0
        assert "3 duplicates" in result.output

    def test_add_json_mode(self):
        mock = _mock_client()
        mock.tasks.add.return_value = AddTasksResult(
            tasks=[Task(id="t1", is_active=True, params={}, status=None, created_at=None)],
            duplicated_count=0,
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "task", "add", "My Squid", "https://a.com"])
        assert result.exit_code == 0


class TestTaskLs:
    def test_list_tasks(self):
        mock = _mock_client()
        mock.tasks.list.return_value = [
            Task(id=FULL_TASK_HASH, is_active=True, params={"url": "https://a.com"},
                 status=None, created_at="2025-01-01T00:00:00Z"),
        ]
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "ls", "My Squid"])
        assert result.exit_code == 0
        assert "https://a.com" in result.output

    def test_list_tasks_pagination(self):
        mock = _mock_client()
        mock.tasks.list.return_value = []
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "ls", "My Squid", "--limit", "10", "--page", "3"])
        assert result.exit_code == 0


class TestTaskShow:
    def test_show_task(self):
        mock = _mock_client()
        mock.tasks.get.return_value = Task(
            id=FULL_TASK_HASH, is_active=True, params={"url": "https://example.com"},
            status=TaskStatus(status="done", total_results=42, total_pages=5,
                              done_reason="completed", has_errors=False),
            created_at="2025-01-01",
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "show", FULL_TASK_HASH])
        assert result.exit_code == 0
        assert "42" in result.output

    def test_show_partial_hash_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "show", "abc123"])
        assert result.exit_code != 0


class TestTaskRm:
    def test_delete_task(self):
        mock = _mock_client()
        mock.tasks.delete.return_value = {"id": FULL_TASK_HASH, "deleted": True}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "rm", FULL_TASK_HASH])
        assert result.exit_code == 0
        assert "Deleted" in result.output

    def test_delete_partial_hash_fails(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "rm", "short"])
        assert result.exit_code != 0


class TestTaskUpload:
    def test_upload_tasks(self, tmp_path):
        csv_file = tmp_path / "tasks.csv"
        csv_file.write_text("url\nhttps://a.com\nhttps://b.com\n")
        mock = _mock_client()
        mock.tasks.upload.return_value = {"id": "upload123"}
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "upload", "My Squid", str(csv_file)])
        assert result.exit_code == 0
        assert "upload123" in result.output


class TestTaskUploadStatus:
    def test_upload_status(self):
        mock = _mock_client()
        mock.tasks.upload_status.return_value = UploadStatus(
            state="completed",
            meta=UploadMeta(valid=100, inserted=95, duplicates=5, invalid=0),
        )
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["task", "upload-status", "upload123"])
        assert result.exit_code == 0
        assert "95" in result.output



import pytest
import json
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.squid import Squid


runner = CliRunner()

SQUIDS = [
    Squid(
        id="squid1abc123def456", name="My Squid", crawler="c1",
        crawler_name="Google Maps", is_active=True, is_ready=True,
        concurrency=1, to_complete=None, last_run_status=None,
        last_run_at=None, total_runs=0, export_unique_results=False,
        params={},
    ),
]

RESULTS = [
    {"id": "r1", "url": "https://a.com", "title": "Page A", "squid": "s1"},
    {"id": "r2", "url": "https://b.com", "title": "Page B", "squid": "s1"},
]


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


def _mock_client(results=None):
    mock = MagicMock()
    mock.squids.list.return_value = SQUIDS
    mock.results.list.return_value = results if results is not None else RESULTS
    return mock


class TestResultsGet:
    def test_get_json_format(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["results", "get", "My Squid"])
        assert result.exit_code == 0
        assert "https://a.com" in result.output

    def test_get_csv_format(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["results", "get", "My Squid", "--format", "csv"])
        assert result.exit_code == 0
        assert "url" in result.output

    def test_get_save_json(self, tmp_path):
        out_file = tmp_path / "out.json"
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["results", "get", "My Squid", "--output", str(out_file)])
        assert result.exit_code == 0
        assert out_file.exists()
        data = json.loads(out_file.read_text())
        assert len(data) == 2

    def test_get_save_csv(self, tmp_path):
        out_file = tmp_path / "out.csv"
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["results", "get", "My Squid", "--format", "csv", "--output", str(out_file)])
        assert result.exit_code == 0
        assert out_file.exists()
        content = out_file.read_text()
        assert "url" in content
        assert "https://a.com" in content

    def test_get_empty_results(self):
        mock = _mock_client(results=[])
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["results", "get", "My Squid", "--format", "csv"])
        assert result.exit_code == 0

    def test_get_with_pagination(self):
        mock = _mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["results", "get", "My Squid", "--page", "2", "--page-size", "10"])
        assert result.exit_code == 0
        mock.results.list.assert_called_once_with(squid="squid1abc123def456", page=2, page_size=10)



from __future__ import annotations

from typing import Optional
import typer

from lobstr_cli import __version__
from lobstrio import LobstrClient
from lobstr_cli.config import get_token
from lobstr_cli.display import set_output_mode, print_error

app = typer.Typer(
    name="lobstr",
    help="CLI for the Lobstr.io scraping API",
    no_args_is_help=True,
    pretty_exceptions_enable=False,
)

# Shared state
_state: dict = {}


def get_client() -> LobstrClient:
    """Get or create the SDK client from global state."""
    if "client" not in _state:
        token = get_token(override=_state.get("token"))
        if not token:
            print_error("No API token. Run: lobstr config set-token <TOKEN>\n  Get your token at https://app.lobstr.io/dashboard/api")
            raise typer.Exit(1)
        client = LobstrClient(token=token)
        if _state.get("verbose"):
            import sys
            _orig_send = client._http._client.send
            def _verbose_send(request, **kwargs):
                resp = _orig_send(request, **kwargs)
                print(f"  {request.method} {request.url} -> {resp.status_code}", file=sys.stderr)
                return resp
            client._http._client.send = _verbose_send
        _state["client"] = client
    return _state["client"]


def _version_callback(value: bool):
    if value:
        typer.echo(f"lobstr {__version__}")
        raise typer.Exit()


@app.callback()
def main(
    json: bool = typer.Option(False, "--json", help="Output raw JSON"),
    token: Optional[str] = typer.Option(None, "--token", envvar="LOBSTR_TOKEN", help="Override API token"),
    verbose: bool = typer.Option(False, "--verbose", help="Show request/response details"),
    quiet: bool = typer.Option(False, "--quiet", help="Suppress non-essential output"),
    version: bool = typer.Option(False, "--version", help="Show version", callback=_version_callback, is_eager=True),
):
    _state["json"] = json
    _state["token"] = token
    _state["verbose"] = verbose
    _state["quiet"] = quiet
    set_output_mode(json_mode=json, quiet=quiet)


# Register command groups
from lobstr_cli.commands.auth import config_app, whoami_app
app.add_typer(config_app, name="config", help="Configuration management")
app.registered_commands.extend(whoami_app.registered_commands)

from lobstr_cli.commands.crawlers import crawlers_app
app.add_typer(crawlers_app, name="crawlers", help="Browse and search crawlers")

from lobstr_cli.commands.squid import squid_app
app.add_typer(squid_app, name="squid", help="Squid management")

from lobstr_cli.commands.task import task_app
app.add_typer(task_app, name="task", help="Task (input URL) management")

from lobstr_cli.commands.run import run_app
app.add_typer(run_app, name="run", help="Run lifecycle management")

from lobstr_cli.commands.go import go_app
app.registered_commands.extend(go_app.registered_commands)

from lobstr_cli.commands.results import results_app
app.add_typer(results_app, name="results", help="Result fetching")

from lobstr_cli.commands.accounts import accounts_app
app.add_typer(accounts_app, name="accounts", help="Account management")

from lobstr_cli.commands.delivery import delivery_app
app.add_typer(delivery_app, name="delivery", help="Delivery configuration")



from __future__ import annotations

import json as json_mod
from typing import Any

from rich.console import Console
from rich.table import Table
from rich.progress import Progress, SpinnerColumn, BarColumn, TextColumn, TimeElapsedColumn

console = Console()
err_console = Console(stderr=True)

# Global state set by CLI callback
_json_mode = False
_quiet_mode = False


def set_output_mode(json_mode: bool = False, quiet: bool = False) -> None:
    global _json_mode, _quiet_mode
    _json_mode = json_mode
    _quiet_mode = quiet


def print_json(data: Any) -> None:
    console.print_json(json_mod.dumps(data, default=str))


def print_error(message: str) -> None:
    if _json_mode:
        print_json({"error": message})
    else:
        err_console.print(f"[bold red]Error:[/] {message}")


def print_success(message: str) -> None:
    if _quiet_mode:
        return
    if _json_mode:
        return
    err_console.print(f"[bold green]{message}[/]")


def print_info(message: str) -> None:
    if _quiet_mode:
        return
    if _json_mode:
        return
    err_console.print(f"[dim]{message}[/]")


def print_table(columns: list[str], rows: list[list[str]], title: str | None = None) -> None:
    table = Table(title=title, show_lines=False)
    for col in columns:
        if col in ("Hash", "Name", "Slug"):
            table.add_column(col, no_wrap=True)
        else:
            table.add_column(col)
    for row in rows:
        table.add_row(*[str(v) for v in row])
    console.print(table)


def print_detail(fields: list[tuple[str, Any]]) -> None:
    table = Table(show_header=False, box=None, padding=(0, 2))
    table.add_column("Field", style="bold cyan")
    table.add_column("Value")
    for name, value in fields:
        table.add_row(name, str(value) if value is not None else "[dim]—[/]")
    console.print(table)


def print_detail_grouped(sections: list[tuple[str | None, list[tuple[str, Any]]]]) -> None:
    """Print detail fields organized into labeled sections.

    Each section is a (title, fields) tuple. Title can be None for untitled sections.
    """

    table = Table(show_header=False, box=None, padding=(0, 2))
    table.add_column("Field", style="bold cyan")
    table.add_column("Value")
    first = True
    for title, fields in sections:
        if not fields:
            continue
        if title:
            if not first:
                table.add_row("", "")
            table.add_row(f"[bold yellow]{title}[/]", "")
        for name, value in fields:
            table.add_row(name, str(value) if value is not None else "[dim]—[/]")
        first = False
    console.print(table)


def make_progress() -> Progress:
    return Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        BarColumn(),
        TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
        TextColumn("{task.completed}/{task.total}"),
        TimeElapsedColumn(),
        console=err_console,
    )



import pytest
from unittest.mock import patch, MagicMock
from typer.testing import CliRunner

from lobstr_cli.cli import app, _state
from lobstrio.models.user import User, Balance


runner = CliRunner()


@pytest.fixture(autouse=True)
def clean_state():
    _state.clear()
    yield
    _state.clear()


class TestSetToken:
    def test_saves_token(self, tmp_path):
        with patch("lobstr_cli.commands.auth.save_token") as mock_save, \
             patch("lobstr_cli.commands.auth.get_config_path", return_value=tmp_path / "config.toml"):
            result = runner.invoke(app, ["config", "set-token", "my-api-token"])
        assert result.exit_code == 0
        mock_save.assert_called_once_with("my-api-token")


class TestShowConfig:
    def test_shows_masked_token(self):
        with patch("lobstr_cli.commands.auth.load_config", return_value={
            "auth": {"token": "abcdefghijklmnop"},
        }), patch("lobstr_cli.commands.auth.get_config_path", return_value="/fake/path"):
            result = runner.invoke(app, ["config", "show"])
        assert result.exit_code == 0
        assert "abcdefgh..." in result.output

    def test_no_token_set(self):
        with patch("lobstr_cli.commands.auth.load_config", return_value={}), \
             patch("lobstr_cli.commands.auth.get_config_path", return_value="/fake/path"):
            result = runner.invoke(app, ["config", "show"])
        assert result.exit_code == 0
        assert "not set" in result.output

    def test_json_mode(self):
        with patch("lobstr_cli.commands.auth.load_config", return_value={
            "auth": {"token": "abcdefghijklmnop"},
        }), patch("lobstr_cli.commands.auth.get_config_path", return_value="/fake"):
            result = runner.invoke(app, ["--json", "config", "show"])
        assert result.exit_code == 0
        assert "abcdefgh..." in result.output

    def test_shows_aliases(self):
        with patch("lobstr_cli.commands.auth.load_config", return_value={
            "auth": {"token": "abcdefghijklmnop"},
            "aliases": {"maps": "abc123def456"},
        }), patch("lobstr_cli.commands.auth.get_config_path", return_value="/fake"):
            result = runner.invoke(app, ["config", "show"])
        assert result.exit_code == 0
        assert "@maps" in result.output


class TestSetAlias:
    def test_creates_alias(self):
        with patch("lobstr_cli.commands.auth.save_alias") as mock_save:
            result = runner.invoke(app, ["config", "set-alias", "myalias", "abc123def456"])
        assert result.exit_code == 0
        mock_save.assert_called_once_with("myalias", "abc123def456")


class TestWhoami:
    def _mock_client(self):
        mock = MagicMock()
        mock.me.return_value = User(
            first_name="John", last_name="Doe",
            email="john@example.com", is_staff=False,
            plan=[{"name": "Pro", "status": "active"}],
        )
        mock.balance.return_value = Balance(
            available=1000, consumed=500, used_slots=2, total_available_slots=5,
        )
        return mock

    def test_whoami_display(self):
        mock = self._mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["whoami"])
        assert result.exit_code == 0
        assert "John Doe" in result.output
        assert "john@example.com" in result.output
        assert "Pro" in result.output
        assert "1000 credits" in result.output

    def test_whoami_json(self):
        mock = self._mock_client()
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["--json", "whoami"])
        assert result.exit_code == 0
        assert "john@example.com" in result.output

    def test_whoami_no_plan(self):
        mock = MagicMock()
        mock.me.return_value = User(
            first_name="", last_name="", email="a@b.com", is_staff=False, plan=[],
        )
        mock.balance.return_value = Balance(available=0, consumed=0, used_slots=0, total_available_slots=0)
        with patch("lobstr_cli.cli.get_client", return_value=mock):
            result = runner.invoke(app, ["whoami"])
        assert result.exit_code == 0
